from PtpUploader.ImageHost.Catbox import CatboxMoe
from PtpUploader.ImageHost.ImgBB import ImgBB
from PtpUploader.ImageHost.PtpImg import PtpImg
from PtpUploader.PtpUploaderException import PtpUploaderException
from PtpUploader.Settings import config


class ImageUploader:
    @staticmethod
    def Upload(logger, imagePath=None, imageUrl=None):
        if (imagePath is None) and (imageUrl is None):
            raise PtpUploaderException(
                "ImageUploader error: both image path and image url are None."
            )

        if (imagePath is not None) and (imageUrl is not None):
            raise PtpUploaderException(
                "ImageUploader error: both image path and image url are given."
            )

        # TODO: fall back to secondary host if the upload to ptpimg wasn't successful. Also start a 1 hour countdown and doesn't use ptpimg till it gets to 0.

        host = None
        if config.image_host.use == "ptpimg":
            host = PtpImg(logger)
        elif config.image_host.use == "imgbb":
            host = ImgBB(logger)
        elif config.image_host.use == "catbox":
            host = CatboxMoe(logger)
        else:
            raise PtpUploaderException(
                "Unknown image host: '%s'." % config.image_host.use
            )

        if imagePath:
            return host.upload_path(imagePath)
        elif imageUrl:
            return host.upload_url(imageUrl)
        else:
            raise PtpUploaderException("No image source specified")



import os
import re
import time

from datetime import timedelta
from pathlib import Path
from typing import List, Tuple

import bencode
import requests

from PtpUploader.MyGlobals import MyGlobals
from PtpUploader.PtpUploaderException import PtpUploaderException


def GetIdxSubtitleLanguages(path: str):
    languages = []

    # id: en, index: 0
    languageRe = re.compile(r"id: ([a-z][a-z]), index: \d+$", re.IGNORECASE)

    # U is needed for "universal" newline support: to handle \r\n as \n.
    with open(path) as pathHandle:
        for line in pathHandle.readlines():
            match = languageRe.match(line)
            if match is not None:
                languages.append(match.group(1))

    return languages


# Supported formats: "100 GB", "100 MB", "100 bytes". (Space is optional.)
# Returns with an integer.
# Returns with 0 if size can't be found.
def GetSizeFromText(text: str):
    text = text.replace(" ", "").replace(
        ",", ""
    )  # For sizes like this: 1,471,981,530bytes
    text = text.replace("GiB", "GB").replace("MiB", "MB")

    matches = re.match("(.+)GB", text)
    if matches is not None:
        return int(float(matches.group(1)) * 1024 * 1024 * 1024)

    matches = re.match("(.+)MB", text)
    if matches is not None:
        return int(float(matches.group(1)) * 1024 * 1024)

    matches = re.match("(.+)bytes", text)
    if matches is not None:
        return int(matches.group(1))

    return 0


def SizeToText(size: int):
    if size < 1024 * 1024 * 1024:
        return "%.2f MiB" % (float(size) / (1024 * 1024))
    return "%.2f GiB" % (float(size) / (1024 * 1024 * 1024))


# timeDifference must be datetime.timedelta.
def TimeDifferenceToText(
    td: timedelta, levels: int = 2, agoText=" ago", noDifferenceText="Just now"
) -> str:
    data = {}
    data["y"], seconds = divmod(int(td.total_seconds()), 31556926)
    # 2629744 seconds = ~1 month (The mean month length of the Gregorian calendar is 30.436875 days.)
    data["mo"], seconds = divmod(seconds, 2629744)
    data["d"], seconds = divmod(seconds, 86400)
    data["h"], seconds = divmod(seconds, 3600)
    data["m"], data["s"] = divmod(seconds, 60)

    time_parts = [f"{round(value)}{name}" for name, value in data.items() if value > 0]
    if time_parts:
        return "".join(time_parts[:levels]) + agoText
    return noDifferenceText


def MakeRetryingHttpGetRequestWithRequests(
    url: str, maximumTries=3, delayBetweenRetriesInSec=10
):
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:34.0) Gecko/20100101 Firefox/34.0"
    }

    while True:
        try:
            result = MyGlobals.session.get(url, headers=headers)
            result.raise_for_status()
            return result
        except requests.exceptions.ConnectionError:
            if maximumTries > 1:
                maximumTries -= 1
                time.sleep(delayBetweenRetriesInSec)
            else:
                raise


def MakeRetryingHttpPostRequestWithRequests(
    url: str, postData, maximumTries=3, delayBetweenRetriesInSec=10
):
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:34.0) Gecko/20100101 Firefox/34.0"
    }

    while True:
        try:
            result = MyGlobals.session.post(url, data=postData, headers=headers)
            result.raise_for_status()
            return result
        except requests.exceptions.ConnectionError:
            if maximumTries > 1:
                maximumTries -= 1
                time.sleep(delayBetweenRetriesInSec)
            else:
                raise


# Path can be a file or a directory. (Obviously.)
def GetPathSize(path) -> int:
    path = Path(path).resolve()
    if path.is_file():
        return path.stat().st_size

    return sum(p.stat().st_size for p in path.rglob("*"))


# Always uses / as path separator.
def GetFileListFromTorrent(torrentPath) -> List[str]:
    with open(torrentPath, "rb") as fh:
        data = bencode.decode(fh.read())
    name = data["info"].get("name", None)
    files = data["info"].get("files", None)

    if files is None:
        return [name]

    fileList = []
    for fileInfo in files:
        path = "/".join(fileInfo["path"])
        fileList.append(path)

    return fileList


def RemoveDisallowedCharactersFromPath(text: str) -> str:
    newText = text

    # These characters can't be in filenames on Windows.
    forbiddenCharacters = r"""\/:*?"<>|"""
    for c in forbiddenCharacters:
        newText = newText.replace(c, "")

    newText = newText.strip()

    if len(newText) > 0:
        return newText
    raise PtpUploaderException("New name for '%s' resulted in empty string." % text)


def ValidateTorrentFile(torrentPath):
    try:
        with open(torrentPath, "rb") as fh:
            bencode.decode(fh.read())
            return True
    except Exception as e:
        raise PtpUploaderException(
            "File '%s' is not a valid torrent." % torrentPath
        ) from e


def GetSuggestedReleaseNameAndSizeFromTorrentFile(torrentPath) -> Tuple[str, int]:
    with open(torrentPath, "rb") as fh:
        data = bencode.decode(fh.read())
    name = data["info"].get("name", None)
    files = data["info"].get("files", None)
    if files is None:
        # It is a single file torrent, remove the extension.
        name, _ = os.path.splitext(name)
        size = data["info"]["length"]
        return name, size
    size = 0
    for file in files:
        size += file["length"]

    return name, size


def DecodeHtmlEntities(html):
    return html.unescape(html)



import errno
import os
import re
import socket
import sys

from datetime import datetime
from pathlib import Path

from django.conf import settings
from django.contrib.staticfiles.handlers import StaticFilesHandler
from django.core.management.base import BaseCommand, CommandError
from django.core.servers.basehttp import WSGIServer, get_internal_wsgi_application
from django.utils import autoreload
from django.utils.regex_helper import _lazy_re_compile
from werkzeug import run_simple
from werkzeug.serving import WSGIRequestHandler, make_ssl_devcert

from PtpUploader import Ptp
from PtpUploader.Job import Supervisor
from PtpUploader.MyGlobals import MyGlobals
from PtpUploader.Settings import Settings, config


naiveip_re = _lazy_re_compile(
    r"""^(?:
(?P<addr>
    (?P<ipv4>\d{1,3}(?:\.\d{1,3}){3}) |         # IPv4 address
    (?P<ipv6>\[[a-fA-F0-9:]+\]) |               # IPv6 address
    (?P<fqdn>[a-zA-Z0-9-]+(?:\.[a-zA-Z0-9-]+)*) # FQDN
):)?(?P<port>\d+)$""",
    re.X,
)


class Command(BaseCommand):
    help = "Starts a lightweight Web server for development."

    # Validation is called explicitly each time the server is reloaded.
    requires_system_checks = []
    stealth_options = ("shutdown_message",)

    default_addr = "127.0.0.1"
    default_addr_ipv6 = "::1"
    default_port = "8000"
    protocol = "http"
    server_cls = WSGIServer

    def add_arguments(self, parser):
        parser.add_argument(
            "addrport", nargs="?", help="Optional port number, or ipaddr:port"
        )
        parser.add_argument(
            "--ipv6",
            "-6",
            action="store_true",
            dest="use_ipv6",
            help="Tells Django to use an IPv6 address.",
        )
        parser.add_argument(
            "--nothreading",
            action="store_false",
            dest="use_threading",
            help="Tells Django to NOT use threading.",
        )
        parser.add_argument(
            "--reload",
            action="store_true",
            dest="use_reloader",
            help="Tells Django to use the auto-reloader.",
        )
        parser.add_argument(
            "--nostatic",
            action="store_false",
            dest="use_static_handler",
            help="Tells Django to NOT automatically serve static files at STATIC_URL.",
        )
        parser.add_argument(
            "--insecure",
            action="store_true",
            dest="insecure_serving",
            help="Allows serving static files even if DEBUG is False.",
        )

    def execute(self, *args, **options):
        if options["no_color"]:
            # We rely on the environment because it's currently the only
            # way to reach WSGIRequestHandler. This seems an acceptable
            # compromise considering `runserver` runs indefinitely.
            os.environ["DJANGO_COLORS"] = "nocolor"
        super().execute(*args, **options)

    def get_handler(self, *args, **options):
        """Return the default WSGI handler for the runner."""
        return StaticFilesHandler(get_internal_wsgi_application())

    def initialize(self, options):
        from django.core.management import call_command
        from django.core.management.commands import migrate
        from django.db import DEFAULT_DB_ALIAS

        from PtpUploader.Job.JobRunningState import JobRunningState
        from PtpUploader.MyGlobals import MyGlobals
        from PtpUploader.ReleaseInfo import ReleaseInfo
        from PtpUploader.Source.SourceFactory import SourceFactory

        Settings.LoadSettings()

        call_command("migrate")

        MyGlobals.InitializeGlobals(Settings.WorkingPath)
        MyGlobals.SourceFactory = SourceFactory()
        MyGlobals.Logger.info("Initializing database...")

        # Reset any possibling interrupted jobs
        for releaseInfo in ReleaseInfo.objects.filter(
            JobRunningState__in=[
                JobRunningState.WaitingForStart,
                JobRunningState.InProgress,
            ]
        ):
            releaseInfo.JobRunningState = JobRunningState.Paused
            releaseInfo.save()

        Settings.VerifyPaths()

    def handle(self, *args, **options):
        self.initialize(options)
        if not settings.DEBUG and not settings.ALLOWED_HOSTS:
            raise CommandError("You must set settings.ALLOWED_HOSTS if DEBUG is False.")

        self.use_ipv6 = options["use_ipv6"]
        if self.use_ipv6 and not socket.has_ipv6:
            raise CommandError("Your Python does not support IPv6.")
        self._raw_ipv6 = False
        if not options["addrport"] and config.web.address:
            options["addrport"] = config.web.address
        if not options["addrport"]:
            self.addr = ""
            self.port = self.default_port
        else:
            m = re.match(naiveip_re, options["addrport"])
            if m is None:
                raise CommandError(
                    '"%s" is not a valid port number '
                    "or address:port pair." % options["addrport"]
                )
            self.addr, _ipv4, _ipv6, _fqdn, self.port = m.groups()
            if not self.port.isdigit():
                raise CommandError("%r is not a valid port number." % self.port)
            if self.addr:
                if _ipv6:
                    self.addr = self.addr[1:-1]
                    self.use_ipv6 = True
                    self._raw_ipv6 = True
                elif self.use_ipv6 and not _fqdn:
                    raise CommandError('"%s" is not a valid IPv6 address.' % self.addr)
        if not self.addr:
            self.addr = self.default_addr_ipv6 if self.use_ipv6 else self.default_addr
            self._raw_ipv6 = self.use_ipv6
        self.run(**options)

    def run(self, **options):
        """Run the server, using the autoreloader if needed."""
        use_reloader = options["use_reloader"]

        if use_reloader:
            autoreload.run_with_reloader(self.inner_run, **options)
        else:
            Ptp.Login()
            MyGlobals.PtpUploader = Supervisor.JobSupervisor()
            MyGlobals.PtpUploader.start()
            self.inner_run(None, **options)

    def ssl_context(self):
        if config.web.ssl.enabled:
            if config.web.ssl.cert and config.web.ssl.key:
                return (config.web.ssl.cert, config.web.ssl.key)
            else:
                return make_ssl_devcert(
                    Path("~/.config/ptpuploader/ssl/").expanduser(), host="localhost"
                )
        else:
            return None

    def inner_run(self, *args, **options):
        # If an exception was silenced in ManagementUtility.execute in order
        # to be raised in the child process, raise it now.
        autoreload.raise_last_exception()

        threading = options["use_threading"]
        # 'shutdown_message' is a stealth option.
        shutdown_message = options.get("shutdown_message", "")
        quit_command = "CTRL-BREAK" if sys.platform == "win32" else "CONTROL-C"

        self.stdout.write("Performing system checks...\n\n")
        self.check(display_num_errors=True)
        # Need to check migrations here, so can't use the
        # requires_migrations_check attribute.
        self.check_migrations()
        now = datetime.now().strftime("%B %d, %Y - %X")
        self.stdout.write(now)
        self.stdout.write(
            (
                "Django version %(version)s, using settings %(settings)r\n"
                "Starting development server at %(protocol)s://%(addr)s:%(port)s/\n"
                "Quit the server with %(quit_command)s."
            )
            % {
                "version": self.get_version(),
                "settings": settings.SETTINGS_MODULE,
                "protocol": self.protocol,
                "addr": "[%s]" % self.addr if self._raw_ipv6 else self.addr,
                "port": self.port,
                "quit_command": quit_command,
            }
        )

        try:
            handler = self.get_handler(*args, **options)
            run_simple(
                self.addr,
                int(self.port),
                handler,
                threaded=threading,
                request_handler=WSGIRequestHandler,
                ssl_context=self.ssl_context(),
            )
        except OSError as e:
            # Use helpful error messages instead of ugly tracebacks.
            ERRORS = {
                errno.EACCES: "You don't have permission to access that port.",
                errno.EADDRINUSE: "That port is already in use.",
                errno.EADDRNOTAVAIL: "That IP address can't be assigned to.",
            }
            try:
                error_text = ERRORS[e.errno]
            except KeyError:
                error_text = e
            self.stderr.write("Error: %s" % error_text)
            # Need to use an OS exit because sys.exit doesn't work in a thread
            os._exit(1)
        except KeyboardInterrupt:
            if shutdown_message:
                self.stdout.write(shutdown_message)
            sys.exit(0)



# Generated by Django 3.2.5 on 2021-10-23 23:05

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ReleaseInfo',
            fields=[
                ('Id', models.AutoField(primary_key=True, serialize=False)),
                ('AnnouncementSourceName', models.TextField(blank=True, default='')),
                ('AnnouncementId', models.TextField(blank=True, default='')),
                ('ReleaseName', models.TextField(blank=True, default='')),
                ('Type', models.TextField(blank=True, default='Feature Film')),
                ('ImdbId', models.TextField(blank=True, default='')),
                ('Directors', models.TextField(blank=True, default='')),
                ('Title', models.TextField(blank=True, default='')),
                ('Year', models.TextField(blank=True, default='')),
                ('Tags', models.TextField(blank=True, default='')),
                ('MovieDescription', models.TextField(blank=True, default='')),
                ('CoverArtUrl', models.TextField(blank=True, default='')),
                ('YouTubeId', models.TextField(blank=True, default='')),
                ('MetacriticUrl', models.TextField(blank=True, default='')),
                ('RottenTomatoesUrl', models.TextField(blank=True, default='')),
                ('Codec', models.TextField(blank=True, default='')),
                ('CodecOther', models.TextField(blank=True, default='')),
                ('Container', models.TextField(blank=True, default='')),
                ('ContainerOther', models.TextField(blank=True, default='')),
                ('ResolutionType', models.TextField(blank=True, default='')),
                ('Resolution', models.TextField(blank=True, default='')),
                ('Source', models.TextField(blank=True, default='')),
                ('SourceOther', models.TextField(blank=True, default='')),
                ('RemasterTitle', models.TextField(blank=True, default='')),
                ('RemasterYear', models.TextField(blank=True, default='')),
                ('JobStartMode', models.IntegerField(default=0)),
                ('JobRunningState', models.IntegerField(default=0)),
                ('FinishedJobPhase', models.IntegerField(default=0)),
                ('Flags', models.IntegerField(default=0)),
                ('ErrorMessage', models.TextField(blank=True, default='')),
                ('PtpId', models.TextField(blank=True, default='')),
                ('PtpTorrentId', models.TextField(blank=True, default='')),
                ('InternationalTitle', models.TextField(blank=True, default='')),
                ('Nfo', models.TextField(blank=True, default='')),
                ('SourceTorrentFilePath', models.TextField(blank=True, default='')),
                ('SourceTorrentInfoHash', models.TextField(blank=True, default='')),
                ('UploadTorrentCreatePath', models.TextField(blank=True, default='')),
                ('UploadTorrentFilePath', models.TextField(blank=True, default='')),
                ('UploadTorrentInfoHash', models.TextField(blank=True, default='')),
                ('ReleaseDownloadPath', models.TextField(blank=True, default='')),
                ('ReleaseUploadPath', models.TextField(blank=True, default='')),
                ('ReleaseNotes', models.TextField(blank=True, default='')),
                ('Screenshots', models.TextField(blank=True, default='')),
                ('LastModificationTime', models.DateTimeField(auto_now=True)),
                ('Size', models.IntegerField(default=0)),
                ('Subtitles', models.TextField(blank=True, default='')),
                ('IncludedFiles', models.TextField(blank=True, default='')),
                ('DuplicateCheckCanIgnore', models.IntegerField(default=0)),
                ('ScheduleTimeUtc', models.DateTimeField()),
            ],
        ),
    ]



import fnmatch
import os
import subprocess
import uuid

from PtpUploader.PtpUploaderException import PtpUploaderException
from PtpUploader.Settings import Settings


# Supported:
# - .rar
# - .001
# - .part01.rar
class Unrar:
    # Because there is no easy way to tell if there will be an overwrite upon extraction, we make a temporary directory into the destination path then move the extracted files out of there.
    @staticmethod
    def Extract(rarPath, destinationPath):
        # Create the temporary folder.
        tempPath = os.path.join(destinationPath, str(uuid.uuid1()))
        if os.path.exists(tempPath):
            raise PtpUploaderException("Temporary path '%s' already exists." % tempPath)
        os.mkdir(tempPath)

        # Extract RAR to the temporary folder.
        args = [Settings.UnrarPath, "x", rarPath, tempPath]
        errorCode = subprocess.call(args)
        if errorCode != 0:
            raise PtpUploaderException(
                "CProcess execution '%s' returned with error code '%s'."
                % (args, errorCode)
            )

        # Move everything out from the temporary folder to its destination.
        files = os.listdir(tempPath)
        for file in files:
            tempFilePath = os.path.join(tempPath, file)
            destinationFilePath = os.path.join(destinationPath, file)
            if os.path.exists(destinationFilePath):
                raise PtpUploaderException(
                    "Can't move file '%s' to '%s' because destination already exists."
                    % (tempFilePath, destinationFilePath)
                )

            os.rename(tempFilePath, destinationFilePath)

        os.rmdir(tempPath)

    @staticmethod
    def IsFirstRar(path):
        path = path.lower()
        if fnmatch.fnmatch(path, "*.001"):
            return True

        if fnmatch.fnmatch(path, "*.rar"):
            if fnmatch.fnmatch(path, "*.part01.rar") or fnmatch.fnmatch(
                path, "*.part001.rar"
            ):
                return True
            if not fnmatch.fnmatch(path, "*.part*.rar"):
                return True

        return False

    @staticmethod
    def GetRars(path):
        entries = os.listdir(path)
        files = []

        for entry in entries:
            if Unrar.IsFirstRar(entry):
                filePath = os.path.join(path, entry)
                if os.path.isfile(filePath):
                    files.append(filePath)

        return files



import subprocess

from PtpUploader.PtpUploaderException import PtpUploaderException


class LibMpv:
    def __init__(self, logger, inputVideoPath):
        self.Logger = logger
        self.InputVideoPath = inputVideoPath
        self.ScaleSize = None

    def MakeScreenshotInPng(self, timeInSeconds, outputPngPath):
        import mpv

        self.Logger.info(
            "Making screenshot with libmpv from '%s' to '%s'."
            % (self.InputVideoPath, outputPngPath)
        )

        args = {
            "audio": "no",
            "screenshot-format": "png",
            "screenshot-png-compression": "9",
            "start": str(int(timeInSeconds)),
            "sub": "no",
            "vf": "lavfi=[scale='max(iw,iw*sar)':'max(ih/sar,ih)']",
            "vo": "null",
        }
        player = mpv.MPV(input_default_bindings=False, input_vo_keyboard=False, **args)

        @player.property_observer("video-frame-info")
        def time_observer(_name, value):
            if value is not None:
                if value["picture-type"] == "I":
                    player.frame_step()
                    img = player.screenshot_raw()
                    img.save(outputPngPath)
                    self.Logger.info("Saved to %s", outputPngPath)
                    player.quit(0)

        player.play(self.InputVideoPath)
        player.wait_for_playback()
        del player



import json
import os

from PtpUploader.Helper import GetFileListFromTorrent
from PtpUploader.Settings import Settings
from PtpUploader.Tool.Unrar import Unrar


class IncludedFileItemState:
    Ignore = 0
    Include = 1


class IncludedFileItem:
    def __init__(self, name):
        self.Name = name  # Path separator is always a "/".
        self.DefaultState = self.__GetDefaultState()
        self.State = self.DefaultState

    def __GetDefaultState(self):
        path = self.Name.lower()

        # Ignore special root directories.
        # !sample is used in HDBits releases.
        # Extras must be uploaded separately on PTP.
        if (
            path.startswith("proof/")
            or path.startswith("sample/")
            or path.startswith("!sample/")
            or path.startswith("samples/")
            or path.startswith("extra/")
            or path.startswith("extras/")
        ):
            return IncludedFileItemState.Ignore

        name = os.path.basename(path)
        if Settings.IsFileOnIgnoreList(name):
            return IncludedFileItemState.Ignore

        if Settings.HasValidVideoExtensionToUpload(
            name
        ) or Settings.HasValidAdditionalExtensionToUpload(name):
            return IncludedFileItemState.Include
        elif Unrar.IsFirstRar(name):
            return IncludedFileItemState.Include
        else:
            return IncludedFileItemState.Ignore

    def IsDefaultIgnored(self):
        return self.DefaultState == IncludedFileItemState.Ignore

    def IsDefaultIncluded(self):
        return self.DefaultState == IncludedFileItemState.Include

    def IsIgnored(self):
        return self.State == IncludedFileItemState.Ignore

    def IsIncluded(self):
        return self.State == IncludedFileItemState.Include


class IncludedFileList:
    def __init__(self):
        self.Files = []  # Contains IncludedFileItems.

    def __GetFile(self, path):
        pathLower = path.lower()
        for file in self.Files:
            if file.Name.lower() == pathLower:
                return file

        return None

    def IsIgnored(self, path):
        file = self.__GetFile(path)
        return file and file.IsIgnored()

    def IsIncluded(self, path):
        file = self.__GetFile(path)
        return file and file.IsIncluded()

    def FromTorrent(self, torrentFilePath):
        self.Files = []
        fileList = GetFileListFromTorrent(torrentFilePath)
        for file in fileList:
            self.Files.append(IncludedFileItem(file))

    def __FromDirectoryInternal(self, path, baseRelativePath):
        entries = os.listdir(path)
        for entry in entries:
            absolutePath = os.path.join(path, entry)
            relativePath = entry
            if len(baseRelativePath) > 0:
                relativePath = baseRelativePath + "/" + entry

            if os.path.isdir(absolutePath):
                self.__FromDirectoryInternal(absolutePath, relativePath)
            elif os.path.isfile(absolutePath):
                self.Files.append(IncludedFileItem(relativePath))

    def FromDirectory(self, path):
        self.Files = []
        self.__FromDirectoryInternal(path, "")

    def ApplyCustomizationFromJson(self, jsonString):
        if len(jsonString) <= 0:
            return

        # Key contains the path, value contains the include state (as bool).
        dictionary = json.loads(jsonString)
        for path, include in list(dictionary.items()):
            file = self.__GetFile(path)
            if file is None:
                file = IncludedFileItem(path)
                self.Files.append(file)

            if include:
                file.State = IncludedFileItemState.Include
            else:
                file.State = IncludedFileItemState.Ignore



import logging

from pathlib import Path

from PtpUploader.PtpUploaderException import PtpUploaderException
from PtpUploader.Settings import config
from PtpUploader.Tool import BdInfo
from PtpUploader.Tool.MediaInfo import MediaInfo, MediaInfoException
from PtpUploader.Tool.ScreenshotMaker import ScreenshotMaker


logger = logging.getLogger(__name__)


class ReleaseDescriptionVideoEntry:
    def __init__(self, info, numberOfScreenshotsToTake=None, bdinfo=None):
        if numberOfScreenshotsToTake is None:
            numberOfScreenshotsToTake = config.uploader.max_screenshots
        self.MediaInfo = info
        self.BdInfo = bdinfo
        self.NumberOfScreenshotsToTake = numberOfScreenshotsToTake
        self.Screenshots = []
        self.ScaleSize = None

    def HaveScreenshots(self):
        return len(self.Screenshots) > 0

    def ToReleaseDescription(self):
        releaseDescription = ""
        if self.BdInfo:
            releaseDescription += self.BdInfo
        else:
            releaseDescription += self.MediaInfo.FormattedMediaInfo

        if self.HaveScreenshots():
            for screenshot in self.Screenshots:
                releaseDescription += "\n\n[img=%s]" % screenshot

        return releaseDescription


class ReleaseDescriptionFormatter:
    def __init__(
        self,
        releaseInfo,
        videoFiles,
        additionalFiles,
        outputImageDirectory,
        makeScreenshots=True,
    ):
        self.ReleaseInfo = releaseInfo
        self.VideoFiles = videoFiles
        self.AdditionalFiles = additionalFiles
        self.OutputImageDirectory = outputImageDirectory
        self.MakeScreenshots = makeScreenshots
        self.VideoEntries = []
        self.MainMediaInfo = None

        self.__GetMediaInfo()
        self.__TakeAndUploadScreenshots()

    def __GetMediaInfoHandleDvdImage(self):
        # Get all IFOs.
        ifos = []
        for file in self.AdditionalFiles:
            file = Path(file)
            if file.suffix == ".IFO":
                mediaInfo = MediaInfo(
                    logger,
                    file,
                    self.ReleaseInfo.GetReleaseUploadPath(),
                )
                ifos.append(mediaInfo)

        # Use the longest by duration.
        ifo = sorted(ifos, key=lambda x: x.DurationInSec, reverse=True)[0]
        if ifo.DurationInSec <= 0:
            raise PtpUploaderException(
                "None of the IFOs have duration. MediaInfo is probably too old."
            )

        if not ifo.Path.name.endswith("_0.IFO"):
            raise PtpUploaderException(
                "Unsupported VIDEO_TS layout. The longest IFO is '%s' with duration '%s'."
                % (ifo.Path, ifo.DurationInSec)
            )

        vobMediaInfo = None
        for vob in sorted(ifo.Path.parent.glob(str(ifo.Path.name[:-5]) + "*.VOB")):
            # Get the next VOB and try to parse its mediainfo
            if str(vob).endswith("0.VOB"):
                continue
            try:
                vobMediaInfo = MediaInfo(
                    logger, vob, self.ReleaseInfo.GetReleaseUploadPath()
                )
                break
            except MediaInfoException:
                pass

        if vobMediaInfo is None:
            raise PtpUploaderException(
                "Unsupported VIDEO_TS layout. Can't find the next VOB for IFO '%s'."
                % ifo.Path
            )
        self.MainMediaInfo = vobMediaInfo
        self.VideoEntries.append(
            ReleaseDescriptionVideoEntry(ifo, numberOfScreenshotsToTake=0)
        )
        self.VideoEntries.append(ReleaseDescriptionVideoEntry(vobMediaInfo))

    def __GetMediaInfoHandleBlurayImage(self):
        # Get all m2ts streams
        m2ts = []
        for file in self.VideoFiles:
            file = Path(file)
            if file.suffix == ".m2ts":
                try:
                    mediaInfo = MediaInfo(
                        logger,
                        file,
                        self.ReleaseInfo.GetReleaseUploadPath(),
                    )
                    m2ts.append(mediaInfo)
                except MediaInfoException:
                    # Discard any non-movie mediainfos
                    pass

        # Use the largest by real size. Mediainfo gets funky with duration/size for
        # m2ts files.
        m2ts = sorted(m2ts, key=lambda x: x.RealFileSize, reverse=True)[0]
        self.MainMediaInfo = m2ts
        path = self.ReleaseInfo.GetReleaseUploadPath()
        if path not in self.ReleaseInfo.BdInfo:
            self.ReleaseInfo.BdInfo[path] = BdInfo.run(path)

        self.VideoEntries.append(
            ReleaseDescriptionVideoEntry(
                m2ts,
                config.uploader.max_screenshots,
                bdinfo=self.ReleaseInfo.BdInfo[path],
            )
        )

    def __GetMediaInfoHandleNonDvdImage(self):
        self.VideoFiles = ScreenshotMaker.SortVideoFiles(self.VideoFiles)
        mediaInfos = MediaInfo.ReadAndParseMediaInfos(
            logger,
            self.VideoFiles,
            self.ReleaseInfo.GetReleaseUploadPath(),
        )
        self.MainMediaInfo = mediaInfos[0]

        # Make less screenshots if there are more than one videos.
        mediaInfoCount = len(mediaInfos)
        numberOfScreenshotsToTake = config.uploader.max_screenshots
        if mediaInfoCount == 2:
            numberOfScreenshotsToTake = 3
        elif mediaInfoCount > 2:
            numberOfScreenshotsToTake = 2

        for i in range(mediaInfoCount):
            self.VideoEntries.append(
                ReleaseDescriptionVideoEntry(mediaInfos[i], numberOfScreenshotsToTake)
            )

    def __GetMediaInfo(self):
        if self.ReleaseInfo.IsDvdImage():
            self.__GetMediaInfoHandleDvdImage()
        elif self.ReleaseInfo.IsBlurayImage():
            self.__GetMediaInfoHandleBlurayImage()
        else:
            self.__GetMediaInfoHandleNonDvdImage()

    def __TakeAndUploadScreenshots(self):
        if not self.MakeScreenshots:
            return

        # Mediainfo will never return the right duration without more
        # magic than it's worth implementing, we'll just use BDInfo
        if self.ReleaseInfo.IsBlurayImage():
            for videoEntry in self.VideoEntries:
                path = str(videoEntry.MediaInfo.Path)
                _, duration = BdInfo.get_longest_playlist(
                    self.ReleaseInfo.GetReleaseUploadPath()
                )
                screenshotMaker = ScreenshotMaker(logger, path)
                videoEntry.ScaleSize = screenshotMaker.GetScaleSize()
                if (
                    path not in self.ReleaseInfo.Screenshots
                    or not self.ReleaseInfo.Screenshots[path]
                ):
                    self.ReleaseInfo.Screenshots[
                        path
                    ] = screenshotMaker.TakeAndUploadScreenshots(
                        self.OutputImageDirectory,
                        duration,
                        videoEntry.NumberOfScreenshotsToTake,
                    )
                    videoEntry.Screenshots = self.ReleaseInfo.Screenshots[path]
            return

        for videoEntry in self.VideoEntries:
            path = str(
                videoEntry.MediaInfo.Path
            )  # Need to make sure any Path objects can be used as keys
            if videoEntry.NumberOfScreenshotsToTake <= 0:
                continue

            screenshotMaker = ScreenshotMaker(logger, path)
            videoEntry.ScaleSize = screenshotMaker.GetScaleSize()

            if (
                path not in self.ReleaseInfo.Screenshots
                or not self.ReleaseInfo.Screenshots[path]
            ):
                self.ReleaseInfo.Screenshots[
                    path
                ] = screenshotMaker.TakeAndUploadScreenshots(
                    self.OutputImageDirectory,
                    videoEntry.MediaInfo.DurationInSec,
                    videoEntry.NumberOfScreenshotsToTake,
                )
                # self.ReleaseInfo.save() # TODO: Rework ReleaseInfoMaker to allow this
            videoEntry.Screenshots = self.ReleaseInfo.Screenshots[path]

    def Format(self, includeReleaseName):
        logger.info("Making release description")
        releaseDescription = ""

        if includeReleaseName:
            releaseDescription = (
                "[size=4][b]%s[/b][/size]\n\n" % self.ReleaseInfo.ReleaseName
            )

        if len(self.ReleaseInfo.ReleaseNotes) > 0:
            releaseDescription += "%s\n\n" % self.ReleaseInfo.ReleaseNotes

        for i, entry in enumerate(self.VideoEntries):
            if i > 0:
                releaseDescription += "\n\n"

            releaseDescription += entry.ToReleaseDescription()

        return releaseDescription

    def GetMainMediaInfo(self):
        return self.MainMediaInfo



# Generated by Django 3.2.5 on 2021-10-25 03:20

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('web', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='releaseinfo',
            name='Trumpable',
            field=models.TextField(blank=True, default=''),
        ),
        migrations.AlterField(
            model_name='releaseinfo',
            name='ScheduleTimeUtc',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
    ]



#!/usr/bin/env python
import os
import sys

from PtpUploader.MyGlobals import MyGlobals
from PtpUploader.Settings import Settings


def run():
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "PtpUploader.web.settings")
    import django

    from django.core.management import execute_from_command_line

    django.setup()
    execute_from_command_line(sys.argv)


if __name__ == "__main__":
    run()



import datetime
import json
import logging

from PtpUploader.PtpUploaderException import PtpUploaderException
from PtpUploader.ReleaseInfo import ReleaseInfo


# Shortcuts for reference
Codecs = ReleaseInfo.CodecChoices
Containers = ReleaseInfo.ContainerChoices
Sources = ReleaseInfo.SourceChoices
Resolutions = ReleaseInfo.ResolutionChoices

logger = logging.getLogger(__name__)


def GetSourceScore(source):
    scores = {
        "VHS": 3,
        "TV": 3,
        "HDTV": 5,
        "WEB": 5,
        # DVD has the same score as HD-DVD and Blu-ray because it must be
        # manually checked if it can co-exists or not.
        "DVD": 7,
        "HD-DVD": 7,
        "Blu-ray": 7,
    }

    return scores.get(source, -1)  # -1 is the default value


# Notes:
# - We treat HD-DVD and Blu-ray as same quality.
# - We treat DVD and Blu-ray rips equally in the standard definition category.
# - We treat H.264 and x264 equally because of the uploading rules: "MP4 can only be trumped by MKV if the use of that container causes problems with video or audio".
# - We treat XviD and DivX as equally irrelevant.
class PtpMovieSearchResult:
    def __init__(self, ptpId, moviePageJsonText):
        self.PtpId = ptpId
        self.ImdbId = ""
        self.ImdbRating = ""
        self.ImdbVoteCount = ""
        self.Torrents = []

        if moviePageJsonText is not None:
            self.__ParseMoviePage(moviePageJsonText)

    def __ParseMoviePageMakeItems(self, itemList, torrent):
        torrent["Id"] = int(torrent["Id"])
        torrent["Size"] = int(torrent["Size"])
        torrent["SourceScore"] = GetSourceScore(torrent["Source"])
        torrent["UploadTime"] = datetime.datetime.strptime(
            torrent["UploadTime"], "%Y-%m-%d %H:%M:%S"
        )
        if "RemasterTitle" not in torrent:
            torrent["RemasterTitle"] = ""
        if "RemasterYear" not in torrent:
            torrent["RemasterYear"] = ""

        fullTitle = f'{torrent["ReleaseName"]} / {torrent["Container"]} / {torrent["Codec"]} / {torrent["Resolution"]}'
        if len(torrent["RemasterTitle"]) > 0:
            fullTitle += " / " + torrent["RemasterTitle"]
            if len(torrent["RemasterYear"]) > 0:
                fullTitle += " (%s)" % torrent["RemasterYear"]
        torrent["FullTitle"] = fullTitle

        itemList.append(torrent)

    def __ParseMoviePage(self, moviePageJsonText):
        moviePageJson = json.loads(moviePageJsonText)

        if moviePageJson["Result"] != "OK":
            raise PtpUploaderException(
                "Unexpected movie page JSON response: '%s'." % moviePageJsonText
            )

        self.ImdbId = moviePageJson.get("ImdbId", "")
        self.ImdbRating = str(moviePageJson.get("ImdbRating", ""))
        self.ImdbVoteCount = str(moviePageJson.get("ImdbVoteCount", ""))

        torrents = moviePageJson["Torrents"]
        if len(torrents) <= 0:
            raise PtpUploaderException(
                "No torrents on movie page 'https://passthepopcorn.me/torrents.php?id=%s'."
                % self.PtpId
            )

        # Get the list of torrents for each section.
        for torrent in torrents:
            self.__ParseMoviePageMakeItems(self.Torrents, torrent)

    def GetLatestTorrent(self):
        return sorted(self.Torrents, key=lambda t: int(t["Id"]), reverse=True)[0]

    def IsReleaseExists(self, release):
        candidates = [t.copy() for t in self.Torrents]  # Semi-shallow copy
        if self.PtpId == "":
            return None
        # Flag un-checkable fields
        if release.Codec == Codecs.Other:
            raise PtpUploaderException(
                "Unsupported codec '%s' for duplicate checking" % release.CodecOther
            )
        if release.Container == Containers.Other:
            raise PtpUploaderException(
                "Unsupported container '%s' for duplicate checking"
                % release.ContainerOther
            )
        if release.Source == Sources.Other:
            raise PtpUploaderException(
                "Unsupported source '%s' for duplicate checking" % release.SourceOther
            )

        # 3.1.3 If literally anything else exists, xvid/divx need manual checking
        if release.Codec in [
            Codecs.XVID,
            Codecs.DIVX,
        ]:
            return candidates[0]
        candidates = [
            t for t in candidates if t["Codec"] not in [Codecs.XVID, Codecs.DIVX]
        ]

        # 4.4.1 One slot per untouched DVD format, and screen them out early
        if release.ResolutionType in ["PAL", "NTSC"]:
            if release.ResolutionType in [t["Resolution"] for t in candidates]:
                return [
                    t for t in candidates if t["Resolution"] == release.ResolutionType
                ][0]
            return None
        candidates = [t for t in candidates if t["Resolution"] not in ["PAL", "NTSC"]]

        for t in candidates:
            # PTP wouldn't let us upload something with the same name anyway
            if t["ReleaseName"] == release.ReleaseName:
                return t
            # Most likely not coincedence
            if t["Size"] == release.Size:
                return t

            # Find any really close duplicates (within 3%)
            if t["Source"] == release.Source and t["Codec"] == release.Codec:
                if abs((release.Size / int(t["Size"])) - 1) * 100 < 3:
                    return t

            # Two slots are available, first check if we can coexist with any of them
            if (
                release.ResolutionType in [Resolutions.Other, "480p"]
                and t["Quality"] == "Standard Definition"
            ):
                if (
                    abs((release.Size / int(t["Size"])) - 1) * 100 < 40
                ):  # 4.1.1.1 40% size difference to be able to coexist
                    return t
            if release.ResolutionType == "576p" and t["Resolution"] == "576p":
                return t

        return None



from django.contrib import admin
from django.shortcuts import redirect
from django.urls import path

from . import views


urlpatterns = [
    path("", lambda r: redirect("/jobs"), name="index"),
    path("jobs", views.jobs, name="jobs"),
    path("ajax/jobs", views.jobs_json, name="jobs_json"),
    path("ajax/localdir", views.local_dir, name="local_dir"),
    path("ajax/getlatest", views.jobs_get_latest, name="jobs_get_latest"),
    path("ajax/create", views.create, name="ajax_create"),
    path("upload", views.edit_job, name="upload"),
    path("settings", views.settings, name="settings"),
    path("upload/bulk", views.bulk_upload, name="bulk_upload"),
    path("movieAvailabilityCheck", views.jobs, name="movieAvailabilityCheck"),
    path("quit", views.jobs, name="quit"),
    path("job/<int:r_id>/log", views.log, name="log"),
    path("job/<int:r_id>/stop", views.stop_job, name="stop_job"),
    path("job/<int:r_id>/start", views.start_job, name="start_job"),
    path("job/<int:r_id>/edit", views.edit_job, name="edit_job"),
    path("job/<int:r_id>/delete/<str:mode>", views.delete_job, name="edit_job"),
    path("admin/", admin.site.urls),
]



import json
import logging

from pathlib import Path
from typing import Dict, List

from PtpUploader.ReleaseInfo import ReleaseInfo
from PtpUploader.Settings import Settings


logger = logging.getLogger(__name__)


def load_json_release(path: Path):
    data: Dict = json.load(path.open())
    release = ReleaseInfo()
    allowed_fields: List[str] = [
        "ImdbId",
        "Title",
        "Year",
        "AnnouncementId",
        "AnnouncementSourceName",
        "CoverArtUrl",
        "Codec",
        "Container",
        "Source",
        "RemasterTitle",
        "Resolution",
    ]
    for k, v in data.items():
        if k in allowed_fields:
            if k == "Source":
                pass  # TODO: properly put things into "other" as needed
            setattr(release, k, v)
    release.JobRunningState = ReleaseInfo.JobState.WaitingForStart
    release.save()
    path.unlink()


def scan_dir():
    path = Path(Settings.GetAnnouncementWatchPath())
    for child in path.iterdir():
        if child.is_file():
            try:
                load_json_release(child)
                continue
            except json.decoder.JSONDecodeError:
                pass
            try:
                load_torrent_release(child)
                continue
            except json.decoder.JSONDecodeError:
                pass



import json
import re

from PtpUploader import Ptp
from PtpUploader.MyGlobals import MyGlobals
from PtpUploader.PtpUploaderException import *


class PtpImdbInfo:
    def __init__(self, imdbId):
        self.ImdbId = imdbId
        self.JsonResponse = ""
        self.JsonMovie = None

    def __LoadmdbInfo(self):
        # Already loaded
        if self.JsonMovie is not None:
            return

        # Get IMDb info through PTP's ajax API used by the site when the user presses the auto fill button.
        result = MyGlobals.session.get(
            "https://passthepopcorn.me/ajax.php?action=torrent_info&imdb=%s"
            % Ptp.NormalizeImdbIdForPtp(self.ImdbId)
        )
        self.JsonResponse = result.text
        Ptp.CheckIfLoggedInFromResponse(result, self.JsonResponse)

        # The response is JSON.
        # [{"title":"Devil's Playground","plot":"As the world succumbs to a zombie apocalypse, Cole a hardened mercenary, is chasing the one person who can provide a cure. Not only to the plague but to Cole's own incumbent destiny. DEVIL'S PLAYGROUND is a cutting edge British horror film that features zombies portrayed by free runners for a terrifyingly authentic representation of the undead","art":false,"year":"2010","director":[{"imdb":"1654324","name":"Mark McQueen","role":null}],"tags":"action, horror","writers":[{"imdb":"1057010","name":"Bart Ruspoli","role":" screenplay"}]}]

        jsonResult = json.loads(self.JsonResponse)
        if len(jsonResult) != 1:
            raise PtpUploaderException(
                "Bad PTP movie info JSON response: array length is not one.\nFull response:\n%s"
                % self.JsonResponse
            )
        if not jsonResult[0]:
            raise PtpUploaderException(
                "Bad PTP movie info JSON response: no movie info. Perhaps the IMDb ID has been moved?"
            )
        self.JsonMovie = jsonResult[0]

    def GetTitle(self):
        self.__LoadmdbInfo()
        title = self.JsonMovie["title"]
        if (title is None) or len(title) == 0:
            raise PtpUploaderException(
                "Bad PTP movie info JSON response: title is empty.\nFull response:\n%s"
                % self.JsonResponse
            )
        return title

    def GetYear(self):
        self.__LoadmdbInfo()
        year = self.JsonMovie["year"]
        if (year is None) or len(year) == 0:
            raise PtpUploaderException(
                "Bad PTP movie info JSON response: year is empty.\nFull response:\n%s"
                % self.JsonResponse
            )
        return year

    def GetMovieDescription(self):
        self.__LoadmdbInfo()
        movieDescription = self.JsonMovie["plot"]
        if movieDescription is None:
            return ""
        return movieDescription

    def GetTags(self):
        self.__LoadmdbInfo()
        tags = self.JsonMovie["tags"]
        if tags is None:
            raise PtpUploaderException(
                "Bad PTP movie info JSON response: tags key doesn't exists.\nFull response:\n%s"
                % self.JsonResponse
            )
        return tags

    def GetCoverArtUrl(self):
        self.__LoadmdbInfo()
        coverArtUrl = self.JsonMovie["art"]
        if coverArtUrl is None:
            raise PtpUploaderException(
                "Bad PTP movie info JSON response: art key doesn't exists.\nFull response:\n%s"
                % self.JsonResponse
            )

        # It may be false... Eg.: "art": false
        if isinstance(coverArtUrl, str):
            # Maximize height in 768 pixels.
            # Example links:
            # http://ia.media-imdb.com/images/M/MV5BMTM2MjE0NTcwNl5BMl5BanBnXkFtZTcwOTM0MDQ1NA@@._V1._SY317_CR1,0,214,317_.jpg
            # http://ia.media-imdb.com/images/M/MV5BMjEwNjQ5NDU4OF5BMl5BanBnXkFtZTYwOTI2NzA5._V1._SY317_CR1,0,214,317_.jpg
            # http://ia.media-imdb.com/images/M/MV5BMzE3NTMwOTk5OF5BMl5BanBnXkFtZTgwODcxNTE1NDE@._V1_UX182_CR0,0,182,268_AL_.jpg
            match = re.match(r"""(.+?\._V1).*\.jpg""", coverArtUrl)
            if match is None:
                return coverArtUrl
            else:
                return match.group(1) + "_SY768_.jpg"
        else:
            return ""


class PtpZeroImdbInfo:
    def __init__(self):
        pass

    def GetTitle(self):
        return ""

    def GetYear(self):
        return ""

    def GetMovieDescription(self):
        return ""

    def GetTags(self):
        return ""

    def GetCoverArtUrl(self):
        return ""



import logging
import os
import shutil

from unidecode import unidecode

from PtpUploader import nfo_parser
from PtpUploader.Helper import GetPathSize
from PtpUploader.IncludedFileList import IncludedFileList
from PtpUploader.PtpUploaderException import PtpUploaderException
from PtpUploader.ReleaseNameParser import ReleaseNameParser
from PtpUploader.Source.SourceBase import SourceBase


logger = logging.getLogger(__name__)


class File(SourceBase):
    UploadDirectoryName = "PTP"

    def __init__(self):
        SourceBase.__init__(self)

        self.Name = "file"
        self.NameInSettings = "FileSource"

    def IsEnabled(self) -> bool:
        return True

    def PrepareDownload(self, _logger, releaseInfo):
        path = releaseInfo.GetReleaseDownloadPath()

        if not os.path.exists(path):
            raise PtpUploaderException(f"Source '{path}' doesn't exist.")

        releaseInfo.Size = GetPathSize(path)

        releaseNameParser = ReleaseNameParser(releaseInfo.ReleaseName)
        releaseNameParser.GetSourceAndFormat(releaseInfo)
        if releaseNameParser.Scene:
            releaseInfo.SetSceneRelease()

    def CheckFileList(self, *_):
        pass

    def IsDownloadFinished(self, _logger, releaseInfo):
        return True

    def GetCustomUploadPath(self, _logger, releaseInfo):
        path = releaseInfo.GetReleaseDownloadPath()
        if releaseInfo.SourceIsAFile():
            # In case of single files the parent directory of the file will be the upload directory.
            return os.path.split(path)[0]
        return os.path.join(
            path, File.UploadDirectoryName, unidecode(releaseInfo.ReleaseName)
        )

    def CreateUploadDirectory(self, releaseInfo):
        if not releaseInfo.SourceIsAFile():
            SourceBase.CreateUploadDirectory(self, releaseInfo)

    def ReadNfo(self, releaseInfo):
        if releaseInfo.SourceIsAFile():
            # Try to read the NFO with the same name as the video file but with nfo extension.
            basePath, fileName = os.path.split(releaseInfo.GetReleaseDownloadPath())
            fileName, _ = os.path.splitext(fileName)
            nfoPath = os.path.join(basePath, fileName) + ".nfo"
            if os.path.isfile(nfoPath):
                releaseInfo.Nfo = nfo_parser.read_nfo(nfoPath)
        else:
            SourceBase.ReadNfo(self, releaseInfo)

    def GetIncludedFileList(self, releaseInfo):
        includedFileList = IncludedFileList()

        path = releaseInfo.GetReleaseDownloadPath()
        if os.path.isdir(path):
            includedFileList.FromDirectory(path)

        return includedFileList

    @staticmethod
    def __DeleteDirectoryWithoutTheUploadDirectory(path):
        if not os.path.isdir(path):
            return

        if not config.uploader.allow_recursive_delete:
            raise PtpUploaderException(
                "Recursive delete requested but functionality is disabled by uploader.allow_recursive_delete setting"
            )

        entries = os.listdir(path)
        for entry in entries:
            if entry == File.UploadDirectoryName:
                continue

            absolutePath = os.path.join(path, entry)

            if os.path.isdir(absolutePath):
                shutil.rmtree(absolutePath)
            elif os.path.isfile(absolutePath):
                os.remove(absolutePath)

    def Delete(self, releaseInfo, torrentClient, deleteSourceData, deleteUploadData):
        # We have to make sure to not to delete source if it is single file because no hard link is being made in this case.
        # Also see how GetCustomUploadPath works.
        sourceIsAFile = None
        path = releaseInfo.GetReleaseDownloadPath()
        if os.path.isdir(path):
            sourceIsAFile = False
        elif os.path.isfile(path):
            sourceIsAFile = True

        if (
            deleteSourceData or deleteUploadData
        ) and not config.uploader.allow_recursive_delete:
            raise PtpUploaderException(
                "Recursive delete requested but functionality is disabled by uploader.allow_recursive_delete setting"
            )

        # Delete source folder without the PTP directory.
        if deleteSourceData:
            if not sourceIsAFile:
                File.__DeleteDirectoryWithoutTheUploadDirectory(
                    releaseInfo.GetReleaseDownloadPath()
                )

        if deleteUploadData:
            # Delete the uploaded torrent file.
            if releaseInfo.UploadTorrentFilePath and os.path.isfile(
                releaseInfo.UploadTorrentFilePath
            ):
                os.remove(releaseInfo.UploadTorrentFilePath)

            # Delete the uploaded torrent from the torrent client.
            if len(releaseInfo.UploadTorrentInfoHash) > 0:
                torrentClient.DeleteTorrent(logger, releaseInfo.UploadTorrentInfoHash)

            # Delete the data of the uploaded torrent.
            # If it is a single file then upload path is its parent directory, so it would be unfortunate to delete. (See GetCustomUploadPath.)
            if not sourceIsAFile and os.path.isdir(releaseInfo.GetReleaseUploadPath()):
                shutil.rmtree(releaseInfo.GetReleaseUploadPath())

        if deleteSourceData and deleteUploadData:
            if sourceIsAFile:
                os.remove(releaseInfo.GetReleaseDownloadPath())

    def GetTemporaryFolderForImagesAndTorrent(self, releaseInfo):
        if releaseInfo.SourceIsAFile():
            return releaseInfo.GetReleaseUploadPath()
        else:
            return os.path.join(
                releaseInfo.GetReleaseDownloadPath(), File.UploadDirectoryName
            )

    def IsSingleFileTorrentNeedsDirectory(self, releaseInfo):
        return not releaseInfo.SourceIsAFile()



import logging
from pathlib import Path

import requests

from guessit import guessit

from PtpUploader.PtpUploaderException import PtpUploaderException
from PtpUploader.Settings import Settings, config


logger = logging.getLogger(__name__)


class ReleaseNameParser:
    def __init__(self, name):
        self.guess = guessit(name, {"enforce_list": True})

        # Simply popping the last tag as a group name wouldn't work because of P2P release with multiple dashes in it:
        # Let Me In 2010 DVDRIP READNFO XViD-T0XiC-iNK

        self.group = ""
        if "release_group" in self.guess:
            self.group = " ".join(self.guess["release_group"])
        self.Scene = self.group in Settings.SceneReleaserGroup
        if not self.Scene and config.uploader.srrdb_screne_check:
            result = None
            try:
                base = Path(name)
                if base.suffix.lstrip(".") in config.uploader.video_files:
                    base = base.stem
                result = requests.get(
                    f"https://api.srrdb.com/v1/search/r:{base}"
                ).json()
                print(result)
                if result["resultsCount"] > 0:
                    self.Scene = True
            except Exception as e:
                if result is not none:
                    logger.debug("srrdb.com result: %s", result)
                logger.error("Failed scene from srrdb.com: %s", e)

    def GetSourceAndFormat(self, releaseInfo):
        if releaseInfo.Codec:
            logger.info(
                "Codec '%s' is already set, not getting from release name."
                % releaseInfo.Codec
            )
        elif "video_codec" in self.guess and len(self.guess["video_codec"]) == 1:
            allowed_codecs = ["XviD", "DivX", "x264", "x265", "H.264", "H.265"]
            for a in allowed_codecs:
                if self.guess["video_codec"][0].lower() == a.lower():
                    if a == "H.264" and "x264" in releaseInfo.ReleaseName.lower():
                        releaseInfo.Codec = "x264"
                    elif a == "H.265" and "x265" in releaseInfo.ReleaseName.lower():
                        releaseInfo.Codec = "x265"
                    else:
                        releaseInfo.Codec = a
                    break
        else:
            raise PtpUploaderException(
                "Can't figure out codec from release name '%s'."
                % releaseInfo.ReleaseName
            )

        if releaseInfo.Source:
            logger.info(
                "Source '%s' is already set, not getting from release name.",
                releaseInfo.Source,
            )
        elif "source" in self.guess and len(self.guess["source"]) == 1:
            allowed_sources = ["DVD", "Blu-ray", "HDTV", "VHS", "TV", "WEB", "HD-DVD"]
            for a in allowed_sources:
                if self.guess["source"][0].lower() == a.lower():
                    releaseInfo.Source = a
                    break
        else:
            raise PtpUploaderException(
                "Can't figure out source from release name '%s'."
                % releaseInfo.ReleaseName
            )

        if releaseInfo.ResolutionType:
            logger.info(
                "Resolution type '%s' is already set, not getting from release name.",
                releaseInfo.ResolutionType,
            )
        elif "screen_size" in self.guess and len(self.guess["screen_size"]) == 1:
            allowed_res = ["576p", "720p", "480p", "1080p", "1080i", "2160p"]
            for a in allowed_res:
                if self.guess["screen_size"][0].lower() == a.lower():
                    releaseInfo.ResolutionType = a
                    break
            if releaseInfo.Source == "DVD" and "other" in self.guess:
                for o in self.guess["other"]:
                    if o in ["NTSC", "PAL"]:
                        releaseInfo.ResolutionType = self.guess["other"]
                        break
        else:
            releaseInfo.ResolutionType = "Other"

        if releaseInfo.Container:
            logger.info(
                "Container '%s' is already set, not getting from release name.",
                releaseInfo.Container,
            )
        elif "container" in self.guess and len(self.guess["container"]) == 1:
            allowed_container = ["AVI", "MKV", "MP4", "VOB IFO", "ISO", "m2ts"]
            for c in allowed_container:
                if c.lower() == self.guess["container"][0].lower():
                    releaseInfo.Container = c
                    break

        if (
            not releaseInfo.RemasterTitle
            and "other" in self.guess
            and "Remux" in self.guess["other"]
        ):
            releaseInfo.RemasterTitle = "Remux"

        if "other" in self.guess and "Reencoded" in self.guess["other"]:
            logger.warning(
                "Re-encoded rip detected from name %s", releaseInfo.ReleaseName
            )

    @staticmethod
    def __IsTagListContainAnythingFromListOfTagList(tagList, listOfTagList):
        # TODO: Confirm this acts as expected
        for listOfTagListElement in listOfTagList:
            if tagList.IsContainsTags(listOfTagListElement.List):
                return str(listOfTagListElement)

        return None

    def values(self):
        return (x for y in self.guess.values() for x in y)

    def IsAllowed(self):
        if self.group in config.source._default.ignore_release_group:
            return "Group '%s' is in your ignore list." % self.group

        if len(Settings.AllowReleaseTag) > 0:
            match = ReleaseNameParser.__IsTagListContainAnythingFromListOfTagList(
                self.values(), Settings.AllowReleaseTag
            )
            if match is None:
                return "Ignored because didn't match your allowed tags setting."

        match = ReleaseNameParser.__IsTagListContainAnythingFromListOfTagList(
            self.values(), Settings.IgnoreReleaseTag
        )
        if match is not None:
            return "'%s' is on your ignore list." % match

        return None



# Three letter codes: ISO 639-2/B
# Two letter codes: ISO 639-1
# http://en.wikipedia.org/wiki/List_of_ISO_639-1_codes
from typing import Optional


subtitle_ids = {
    3: ["english", "eng", "en", "english (cc)", "english - sdh"],
    4: ["spanish", "spa", "es"],
    5: ["french", "fre", "fr"],
    6: ["german", "ger", "de"],
    7: ["russian", "rus", "ru"],
    8: ["japanese", "jpn", "ja"],
    9: ["dutch", "dut", "nl"],
    10: ["danish", "dan", "da"],
    11: ["swedish", "swe", "sv"],
    12: ["norwegian", "nor", "no"],
    13: ["romanian", "rum", "ro"],
    14: ["chinese", "chi", "zh", "chinese (simplified)", "chinese (traditional)"],
    15: ["finnish", "fin", "fi"],
    16: ["italian", "ita", "it"],
    17: ["polish", "pol", "pl"],
    18: ["turkish", "tur", "tr"],
    19: ["korean", "kor", "ko"],
    20: ["thai", "tha", "th"],
    21: ["portuguese", "por", "pt"],
    22: ["arabic", "ara", "ar"],
    23: ["croatian", "hrv", "hr", "scr"],
    24: ["hungarian", "hun", "hu"],
    25: ["vietnamese", "vie", "vi"],
    26: ["greek", "gre", "el"],
    28: ["icelandic", "ice", "is"],
    29: ["bulgarian", "bul", "bg"],
    30: ["czech", "cze", "cz", "cs"],
    31: ["serbian", "srp", "sr", "scc"],
    34: ["ukrainian", "ukr", "uk"],
    37: ["latvian", "lav", "lv"],
    38: ["estonian", "est", "et"],
    39: ["lithuanian", "lit", "lt"],
    40: ["hebrew", "heb", "he"],
    41: ["hindihin", "hi"],
    42: ["slovak", "slo", "sk"],
    43: ["slovenian", "slv", "sl"],
    44: ["no subtitles"],
    47: ["indonesian", "ind", "id"],
    49: ["brazilian portuguese", "brazilian", "portuguese-br"],
    50: ["english - forced", "english (forced)"],
    51: ["english intertitles", "english (intertitles)", "english - intertitles"],
    52: ["persian", "fa", "far"],
}


def get_id(lang: str) -> Optional[int]:
    lang = lang.lower()
    for sub_id, names in subtitle_ids.items():
        if lang in names:
            return sub_id
    return None



import datetime
import logging
import os

from pathlib import Path
from typing import Iterator, List

from django.db import models
from django.utils import timezone
from unidecode import unidecode

from PtpUploader.Job.FinishedJobPhase import FinishedJobPhase
from PtpUploader.Job.JobStartMode import JobStartMode
from PtpUploader.MyGlobals import MyGlobals
from PtpUploader.PtpUploaderException import PtpUploaderException
from PtpUploader.Settings import Settings


class ReleaseInfo(models.Model):
    # pylint: disable=too-many-public-methods, too-many-instance-attributes
    class Meta:
        app_label = "web"

    class JobState(models.IntegerChoices):
        WaitingForStart = 0, "Waiting for start"
        InProgress = 1, "In progress"
        Paused = 2
        Finished = 3
        Failed = 4
        Ignored = 5
        Ignored_AlreadyExists = 6, "Ignored, already exists"
        Ignored_Forbidden = 7, "Ignored, forbidden"
        Ignored_MissingInfo = 8, "Ignored, missing info"
        Ignored_NotSupported = 9, "Ignored, not supported"
        DownloadedAlreadyExists = 10, "Downloaded, already exists"
        Scheduled = 11
        InDownload = 12, "Downloading"

    class TypeChoices(models.TextChoices):
        feature = ("Feature Film", "Feature Film")
        short = ("Short Film", "Short Film")
        miniseries = ("Miniseries", "Miniseries")
        standup = ("Stand-up Comedy", "Stand-up Comedy")
        concert = ("Concert", "Concert")
        live = ("Live Performance", "Live Performance")
        collection = ("Movie Collection", "Movie Collection")

    class CodecChoices(models.TextChoices):
        XVID = ("XviD", "XviD")
        DIVX = ("DivX", "DivX")
        H_264 = ("H.264", "H.264")
        X264 = ("x264", "x264")
        H_265 = ("H.265", "H.265")
        X265 = ("x265", "x265")
        DVD5 = ("DVD5", "DVD5")
        DVD9 = ("DVD9", "DVD9")
        BD25 = ("BD25", "BD25")
        BD50 = ("BD50", "BD50")
        BD66 = ("BD66", "BD66")
        BD100 = ("BD100", "BD100")
        Other = ("Other", "Other")
        __empty__ = "---"

    class ContainerChoices(models.TextChoices):
        AVI = ("AVI", "AVI")
        MPG = ("MPG", "MPG")
        MKV = ("MKV", "MKV")
        MP4 = ("MP4", "MP4")
        VOB_IFO = ("VOB IFO", "VOB IFO")
        ISO = ("ISO", "ISO")
        M2TS = ("m2ts", "m2ts")
        Other = ("Other", "Other")
        __empty__ = "---"

    class ResolutionChoices(models.TextChoices):
        NTSC = ("NTSC", "NTSC")
        PAL = ("PAL", "PAL")
        r480p = ("480p", "480p")
        r576p = ("576p", "576p")
        r720p = ("720p", "720p")
        r1080i = ("1080i", "1080i")
        r1080p = ("1080p", "1080p")
        r2160p = ("2160p", "2160p")
        Other = ("Other", "Other")
        __empty__ = "---"

    class SourceChoices(models.TextChoices):
        Bluray = ("Blu-ray", "Blu-ray")
        DVD = ("DVD", "DVD")
        WEB = ("WEB", "WEB")
        HDDVD = ("HD-DVD", "HD-DVD")
        HDTV = ("HDTV", "HDTV")
        TV = ("TV", "TV")
        VHS = ("VHS", "VHS")
        Other = ("Other", "Other")
        __empty__ = "---"

    class TrumpableReasons(models.IntegerChoices):
        NO_ENGLISH_SUBS = 14
        HARDCODED_SUBS = 4

    objects: models.manager.Manager  # helps to find some typing errors

    Id = models.AutoField(primary_key=True)

    # Announcement
    AnnouncementSourceName = models.TextField(blank=True, default="")
    AnnouncementId = models.TextField(blank=True, default="")
    ReleaseName = models.TextField(blank=True, default="")

    # For PTP
    Type = models.TextField(
        blank=True, default=TypeChoices.feature, choices=TypeChoices.choices
    )
    ImdbId = models.TextField(blank=True, default="")
    Directors = models.TextField(blank=True, default="")
    Title = models.TextField(blank=True, default="")
    Year = models.TextField(blank=True, default="")
    MovieDescription = models.TextField(blank=True, default="")
    CoverArtUrl = models.TextField(blank=True, default="")
    YouTubeId = models.TextField(blank=True, default="")
    MetacriticUrl = models.TextField(blank=True, default="")
    RottenTomatoesUrl = models.TextField(blank=True, default="")
    Codec = models.TextField(blank=True, default="")
    CodecOther = models.TextField(blank=True, default="")
    Container = models.TextField(blank=True, default="")
    ContainerOther = models.TextField(blank=True, default="")
    ResolutionType = models.TextField(blank=True, default="")
    Resolution = models.TextField(blank=True, default="")
    Source = models.TextField(blank=True, default="")
    SourceOther = models.TextField(blank=True, default="")
    RemasterTitle = models.TextField(blank=True, default="")
    RemasterYear = models.TextField(blank=True, default="")

    # Other
    JobStartMode = models.IntegerField(default=JobStartMode.Automatic)
    JobRunningState = models.IntegerField(
        choices=JobState.choices, default=JobState.WaitingForStart
    )
    FinishedJobPhase = models.IntegerField(default=0)
    Tags = models.TextField(blank=True, default="")
    ErrorMessage = models.TextField(blank=True, default="")
    PtpId = models.TextField(blank=True, default="")
    PtpTorrentId = models.TextField(blank=True, default="")
    InternationalTitle = models.TextField(blank=True, default="")
    Nfo = models.TextField(blank=True, default="")
    SourceTorrentFilePath = models.TextField(blank=True, default="")
    SourceTorrentInfoHash = models.TextField(blank=True, default="")
    UploadTorrentCreatePath = models.TextField(blank=True, default="")
    UploadTorrentFilePath = models.TextField(blank=True, default="")
    UploadTorrentInfoHash = models.TextField(blank=True, default="")
    ReleaseDownloadPath = models.TextField(blank=True, default="")
    ReleaseUploadPath = models.TextField(blank=True, default="")
    ReleaseNotes = models.TextField(blank=True, default="")
    Screenshots = models.JSONField(blank=True, default=dict)
    # Mediainfo is pretty quick even for large files, BDInfo can be much rougher (and
    # produces less data), so let's cache it.
    BdInfo = models.JSONField(blank=True, default=dict)
    LastModificationTime = models.DateTimeField(auto_now=True)
    Size = models.BigIntegerField(default=0)
    Subtitles = models.JSONField(blank=True, default=list)  # CSV of subtitle IDs
    IncludedFiles = models.TextField(blank=True, default="")
    DuplicateCheckCanIgnore = models.IntegerField(default=0)
    ScheduleTime = models.DateTimeField(
        default=datetime.datetime.fromtimestamp(0, timezone.get_default_timezone()),
        null=True,
    )
    Trumpable: List[int] = models.JSONField(
        blank=True, default=list
    )  # CSV of trump IDs
    SpecialRelease = models.BooleanField(default=False)
    # Release made by a scene group.
    SceneRelease = models.BooleanField(default=False)
    # If set, then it overrides the value returned by SourceBase.IsSingleFileTorrentNeedsDirectory.
    ForceDirectorylessSingleFileTorrent = models.BooleanField(default=False)
    # If this is set then the job will be the next processed job and the download will start regardless the number of maximum parallel downloads set for the source.
    StartImmediately = models.BooleanField(default=False)
    # Job will be stopped before uploading.
    StopBeforeUploading = models.BooleanField(default=False)
    OverrideScreenshots = models.BooleanField(default=False)
    PersonalRip = models.BooleanField(default=False)

    def __init__(self, *args, **kwargs):
        # <<< These are the required fields needed for an upload to PTP.
        super().__init__(*args, **kwargs)

        self.SceneAccessDownloadUrl = ""  # Temporary store for FunFile.
        self.ImdbRating = ""  # Not saved in the database.
        self.ImdbVoteCount = ""  # Not saved in the database.
        self.JobStartTimeUtc = datetime.datetime.utcnow()

    def IsUserCreatedJob(self):
        return self.JobStartMode in [JobStartMode.Manual, JobStartMode.ManualForced]

    def IsForceUpload(self):
        return self.JobStartMode == JobStartMode.ManualForced

    def GetDirectors(self):
        if len(self.Directors) > 0:
            return self.Directors.split(", ")
        return []

    def SetDirectors(self, names):
        for name in names:
            if name.find(",") != -1:
                raise PtpUploaderException(
                    "Director name '%s' contains a comma." % name
                )

        self.Directors = ", ".join(names)

    def SetSubtitles(self, sub_ids: Iterator[int]):
        self.Subtitles = sub_ids

    def IsSceneRelease(self):
        return self.SceneRelease

    def SetSceneRelease(self):
        self.SceneRelease = True

    def IsHighDefinition(self):
        return self.ResolutionType in ["720p", "1080i", "1080p"]

    def IsStandardDefinition(self):
        return (not self.IsHighDefinition()) and (not self.IsUltraHighDefinition())

    def IsUltraHighDefinition(self):
        return self.ResolutionType in ["4K", "2160p"]

    def IsRemux(self):
        return "Remux" in str(self.RemasterTitle)

    def IsDvdImage(self):
        return self.Codec in ["DVD5", "DVD9"]

    def IsBlurayImage(self):
        return str(self.Codec).startswith("BD")

    def CanEdited(self):
        return self.JobRunningState not in [
            self.JobState.WaitingForStart,
            self.JobState.Scheduled,
            self.JobState.InProgress,
            self.JobState.InDownload,
            self.JobState.Finished,
        ]

    def IsReleaseNameEditable(self):
        return self.CanEdited() and not self.IsJobPhaseFinished(
            FinishedJobPhase.Download_CreateReleaseDirectory
        )

    def CanResumed(self):
        return self.CanEdited()

    def CanStopped(self):
        return self.JobRunningState in [
            self.JobState.WaitingForStart,
            self.JobState.Scheduled,
            self.JobState.InDownload,
            self.JobState.InProgress,
        ]

    def CanDeleted(self):
        return not self.CanStopped()

    def IsJobPhaseFinished(self, jobPhase):
        return (self.FinishedJobPhase & jobPhase) != 0

    def SetJobPhaseFinished(self, jobPhase):
        self.FinishedJobPhase |= jobPhase

    # Eg.: "working directory/release/Dark.City.1998.Directors.Cut.720p.BluRay.x264-SiNNERS/"
    def GetReleaseRootPath(self):
        return Path(Settings.WorkingPath, "release", self.ReleaseName)

    def SourceIsAFile(self):
        return Path(self.GetReleaseDownloadPath()).is_file()

    # Eg.: "working directory/release/Dark.City.1998.Directors.Cut.720p.BluRay.x264-SiNNERS/download/"
    def GetReleaseDownloadPath(self):
        if self.ReleaseDownloadPath:
            return self.ReleaseDownloadPath
        return os.path.join(self.GetReleaseRootPath(), "download")

    # Eg.: "working directory/release/Dark.City.1998.Directors.Cut.720p.BluRay.x264-SiNNERS/upload/Dark.City.1998.Directors.Cut.720p.BluRay.x264-SiNNERS/"
    # It must contain the final release name because of mktor.
    def GetReleaseUploadPath(self):
        if self.ReleaseUploadPath:
            return self.ReleaseUploadPath
        return os.path.join(
            self.GetReleaseRootPath(), "upload", unidecode(self.ReleaseName)
        )

    def IsZeroImdbId(self):
        return self.ImdbId == "0"

    @property
    def AnnouncementSource(self):
        return MyGlobals.SourceFactory.GetSource(self.AnnouncementSourceName)

    def logger(self, logger=None):
        if logger is None:
            logger = logging.getLogger(f"PtpUploader.ReleaseInfo.Release{self.Id}")
        logger_with_id = logging.LoggerAdapter(logger, {"release_id": self.Id})
        return logger_with_id



class PtpUploaderMessageBase:
    pass


class PtpUploaderMessageStartJob(PtpUploaderMessageBase):
    def __init__(self, releaseInfoId):
        self.ReleaseInfoId = releaseInfoId


class PtpUploaderMessageStopJob(PtpUploaderMessageBase):
    def __init__(self, releaseInfoId):
        self.ReleaseInfoId = releaseInfoId


class PtpUploaderMessageDeleteJob(PtpUploaderMessageBase):
    def __init__(self, releaseInfoId, mode):
        self.ReleaseInfoId = releaseInfoId
        self.mode = mode


class PtpUploaderMessageNewAnnouncementFile(PtpUploaderMessageBase):
    def __init__(self, announcementFilePath):
        self.AnnouncementFilePath = announcementFilePath


class PtpUploaderMessageQuit(PtpUploaderMessageBase):
    pass



# Generated by Django 3.2.5 on 2021-10-27 19:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('web', '0002_auto_20211025_0320'),
    ]

    operations = [
        migrations.AlterField(
            model_name='releaseinfo',
            name='JobRunningState',
            field=models.IntegerField(choices=[(0, 'Waiting for start'), (1, 'In progress'), (2, 'Paused'), (3, 'Finished'), (4, 'Failed'), (5, 'Ignored'), (6, 'Ignored, already exists'), (7, 'Ignored, forbidden'), (8, 'Ignored, missing info'), (9, 'Ignored, not supported'), (10, 'Downloaded, already exists'), (11, 'Scheduled'), (12, 'Downloading')], default=0),
        ),
    ]



import json
import re
import time
import traceback

from PtpUploader.MyGlobals import MyGlobals
from PtpUploader.PtpMovieSearchResult import PtpMovieSearchResult
from PtpUploader.PtpUploaderException import *
from PtpUploader.Settings import Settings


def __LoginInternal():
    if not Settings.PtpUserName:
        raise PtpUploaderInvalidLoginException(
            "Couldn't log in to PTP. Your user name is not specified."
        )

    if not Settings.PtpPassword:
        raise PtpUploaderInvalidLoginException(
            "Couldn't log in to PTP. Your password is not specified."
        )

    # Get the pass key from the announce URL.
    passKey = re.match(
        r"https?://please\.passthepopcorn\.me:?\d*/(.+)/announce",
        Settings.PtpAnnounceUrl,
    )
    if passKey is None:
        raise PtpUploaderInvalidLoginException(
            "Couldn't log in to PTP. Pass key not found in the announce URL."
        )
    passKey = passKey.group(1)

    MyGlobals.Logger.info("Logging in to PTP.")
    logged_in = False
    try:
        if "passthepopcorn.me" in MyGlobals.session.cookies.list_domains():
            response = MyGlobals.session.get("https://passthepopcorn.me/upload.php")
            CheckIfLoggedInFromResponse(response, response.text)
            Settings.AntiCsrfToken = re.search(
                r'data-AntiCsrfToken="(.*)"', response.text
            ).group(1)
            logged_in = True
    except PtpUploaderException:
        pass

    if not logged_in:
        postData = {
            "username": Settings.PtpUserName,
            "password": Settings.PtpPassword,
            "passkey": passKey,
            "keeplogged": "1",
        }
        response = MyGlobals.session.post(
            "https://passthepopcorn.me/ajax.php?action=login", data=postData
        ).text

        print(response)
        jsonLoad = None
        try:
            jsonLoad = json.loads(response)
        except (Exception, ValueError) as e:
            raise PtpUploaderInvalidLoginException(
                "Got exception while loading JSON login response from PTP. Response: '%s'."
                % response
            ) from e

        if jsonLoad is None:
            raise PtpUploaderInvalidLoginException(
                "Got bad JSON response from PTP while trying to log in. Response: '%s'."
                % response
            )

        if jsonLoad["Result"] == "TfaRequired":
            postData["TfaCode"] = input("Enter 2FA auth code:")
            postData["TfaType"] = "normal"
            response = MyGlobals.session.post(
                "https://passthepopcorn.me/ajax.php?action=login", data=postData
            ).text
            jsonLoad = json.loads(response)
            if jsonLoad["Result"] != "Ok":
                raise PtpUploaderInvalidLoginException(
                    "Failed to login to PTP. Probably due to a bad 2FA auth code. Response: '%s'."
                    % jsonLoad
                )
            Settings.AntiCsrfToken = jsonLoad["AntiCsrfToken"]
        elif jsonLoad["Result"] != "Ok":
            raise PtpUploaderInvalidLoginException(
                "Failed to login to PTP. Probably due to the bad user name, password or pass key. Response: '%s'."
                % jsonLoad
            )
        else:
            Settings.AntiCsrfToken = jsonLoad["AntiCsrfToken"]
        MyGlobals.SaveCookies()


def Login():
    maximumTries = 1

    while True:
        try:
            __LoginInternal()
            return
        except PtpUploaderInvalidLoginException:
            raise
        except Exception:
            if maximumTries > 1:
                maximumTries -= 1
                time.sleep(30)  # Wait 30 seconds and retry.
            else:
                raise


def __CheckIfLoggedInFromResponseLogResponse(result, responseBody: str):
    MyGlobals.Logger.debug("MSG: %s", result.reason)
    MyGlobals.Logger.debug("CODE: %s", result.status_code)
    MyGlobals.Logger.debug("URL: %s", result.url)
    MyGlobals.Logger.debug("HEADERS: %s", result.headers)
    MyGlobals.Logger.debug("STACK: %s", traceback.format_stack())
    MyGlobals.Logger.debug("RESPONSE BODY: %s", responseBody)


def CheckIfLoggedInFromResponse(result, responseBody: str):
    if responseBody.find("""<a href="login.php?act=recover">""") != -1:
        __CheckIfLoggedInFromResponseLogResponse(result, responseBody)
        raise PtpUploaderException(
            "Looks like you are not logged in to PTP. Probably due to the bad user name or password."
        )
    if "Your popcorn quota has been reached, come back later!" in responseBody:
        raise PtpUploaderException(
            "Your PTP request/popcorn quota has been reached, try again later"
        )


# PTP expects 7 character long IMDb IDs.
# E.g.: it can't find movie with IMDb ID 59675, only with 0059675. (IMDb redirects to the latter.)


def NormalizeImdbIdForPtp(imdbId):
    return imdbId.rjust(7, "0")


# ptpId must be a valid id
# returns with PtpMovieSearchResult


def GetMoviePageOnPtp(logger, ptpId):
    logger.info("Getting movie page for PTP id '%s'." % ptpId)

    result = MyGlobals.session.get(
        "https://passthepopcorn.me/torrents.php?id=%s&json=1" % ptpId
    )
    response = result.text
    CheckIfLoggedInFromResponse(result, response)

    if response.find("<h2>Error 404</h2>") != -1:
        raise PtpUploaderException("Movie with PTP id '%s' doesn't exists." % ptpId)

    return PtpMovieSearchResult(ptpId, response)


# imdbId: IMDb id. Eg.: 0137363 for http://www.imdb.com/title/tt0137363/
# returns with PtpMovieSearchResult


def GetMoviePageOnPtpByImdbId(logger, imdbId):
    logger.info("Trying to find movie with IMDb id '%s' on PTP." % imdbId)

    result = MyGlobals.session.get(
        "https://passthepopcorn.me/torrents.php?imdb=%s&json=1"
        % NormalizeImdbIdForPtp(imdbId)
    )
    response = result.text
    CheckIfLoggedInFromResponse(result, response)

    # If there is a movie: result.url = https://passthepopcorn.me/torrents.php?id=28577
    # If there is no movie: result.url = https://passthepopcorn.me/torrents.php?imdb=1535492
    match = re.match(r".*?passthepopcorn\.me/torrents\.php\?id=(\d+)", result.url)
    if match is not None:
        ptpId = match.group(1)
        url = result.url
        url = url.replace("&json=1", "")
        logger.info(f"Movie with IMDb id '{imdbId}' exists on PTP at '{url}'.")
        return PtpMovieSearchResult(ptpId, response)
    elif (
        response.find("<h2>Error 404</h2>") != -1
    ):  # For some deleted movies PTP return with this error.
        logger.info(
            "Movie with IMDb id '%s' doesn't exist on PTP. (Got error 404.)" % imdbId
        )
        return PtpMovieSearchResult(ptpId="", moviePageJsonText=None)
    elif response.find("<h2>Your search did not match anything.</h2>") != -1:
        logger.info("Movie with IMDb id '%s' doesn't exist on PTP." % imdbId)
        return PtpMovieSearchResult(ptpId="", moviePageJsonText=None)
    else:  # Multiple movies with the same IMDb id.
        raise PtpUploaderException(
            "There are multiple movies on PTP with IMDb id '%s'." % imdbId
        )


def __UploadMovieGetParamsCommon(releaseInfo, releaseDescription):
    paramList = {
        "type": releaseInfo.Type,
        "remaster_year": releaseInfo.RemasterYear,
        "remaster_title": releaseInfo.RemasterTitle,
        "codec": "Other",  # Sending the codec as custom.
        "other_codec": releaseInfo.Codec,
        "container": "Other",  # Sending the container as custom.
        "other_container": releaseInfo.Container,
        "resolution": releaseInfo.ResolutionType,
        "other_resolution": releaseInfo.Resolution,
        "source": "Other",  # Sending the source as custom.
        "other_source": releaseInfo.Source,
        "release_desc": releaseDescription,
        "nfo_text": releaseInfo.Nfo,
        "AntiCsrfToken": Settings.AntiCsrfToken,
        "subtitles[]": releaseInfo.Subtitles,
        "trumpable[]": releaseInfo.Trumpable,
    }

    if releaseInfo.Source == "Other":
        paramList["other_source"]: releaseInfo.SourceOther

    # personal rip only needed if it is specified
    if releaseInfo.PersonalRip:
        paramList.update({"internalrip": "on"})

    # scene only needed if it is specified
    if releaseInfo.SceneRelease:
        paramList.update({"scene": "on"})
    # other category is only needed if it is specified
    if releaseInfo.SpecialRelease:
        paramList.update({"special": "on"})
    # remaster is only needed if it is specified
    if len(releaseInfo.RemasterYear) > 0 or len(releaseInfo.RemasterTitle) > 0:
        paramList.update({"remaster": "on"})
    return paramList


def __UploadMovieGetParamsForAddFormat(ptpId):
    groupId = ("groupid", ptpId)
    return [groupId]


def __UploadMovieGetParamsForNewMovie(releaseInfo):
    params = {
        "title": releaseInfo.Title,
        "year": releaseInfo.Year,
        "image": releaseInfo.CoverArtUrl,
        "tags": releaseInfo.Tags,
        "album_desc": releaseInfo.MovieDescription,
        "trailer": releaseInfo.YouTubeId,
    }

    # Add the IMDb ID.
    if releaseInfo.ImdbId == "0":
        params.update({"imdb": "0"})
    else:
        params.update({"imdb": NormalizeImdbIdForPtp(releaseInfo.ImdbId)})
    # Add the directors.
    # These needs to be added in order because of the "importance" field follows them.
    for d in releaseInfo.GetDirectors():
        params.update({"artist[]": d})
        params.update({"importance[]": "1"})

    return params


# Returns with the auth key.


def UploadMovie(logger, releaseInfo, torrentPath, releaseDescription):
    url = ""
    paramList = __UploadMovieGetParamsCommon(releaseInfo, releaseDescription)

    # We always use HTTPS for uploading because if "Force HTTPS" is enabled in the profile then the HTTP upload is not working.
    if releaseInfo.PtpId:
        logger.info(
            "Uploading torrent '%s' to PTP as a new format for 'https://passthepopcorn.me/torrents.php?id=%s'."
            % (torrentPath, releaseInfo.PtpId)
        )
        url = "https://passthepopcorn.me/upload.php?groupid=%s" % releaseInfo.PtpId
        paramList.update(__UploadMovieGetParamsForAddFormat(releaseInfo.PtpId))
    else:
        logger.info("Uploading torrent '%s' to PTP as a new movie." % torrentPath)
        url = "https://passthepopcorn.me/upload.php"
        paramList.update(__UploadMovieGetParamsForNewMovie(releaseInfo))

    # Add the torrent file, passing a fake string for the filename, since it's not necessary anyway
    with open(torrentPath, "rb") as torrentHandle:
        files = {
            "file_input": (
                "placeholder.torrent",
                torrentHandle,
                "application/x-bittorent",
            )
        }
        result = MyGlobals.session.post(url, data=paramList, files=files)
    response = result.text
    CheckIfLoggedInFromResponse(result, response)

    # If the repsonse contains our announce url then we are on the upload page and the upload wasn't successful.
    if response.find(Settings.PtpAnnounceUrl) != -1:
        # Get the error message.
        # <div class="alert alert--error text--center">No torrent file uploaded, or file is empty.</div>
        errorMessage = ""
        match = re.search(
            r"""<div class="alert alert--error.*?>(.+?)</div>""", response
        )
        if match is not None:
            errorMessage = match.group(1)

        raise PtpUploaderException(
            "Upload to PTP failed: '%s' (%s). (We are still on the upload page.)"
            % (errorMessage, result.status_code)
        )

    # URL format in case of successful upload: https://passthepopcorn.me/torrents.php?id=9329&torrentid=91868
    match = re.match(
        r".*?passthepopcorn\.me/torrents\.php\?id=(\d+)&torrentid=(\d+)", result.url
    )
    if match is None:
        raise PtpUploaderException(
            "Upload to PTP failed: result URL '%s' (%s) is not the expected one."
            % (result.url, result.status_code)
        )

    ptpId = match.group(1)
    releaseInfo.PtpTorrentId = match.group(2)

    if not releaseInfo.PtpId:
        releaseInfo.PtpId = ptpId


def SendPrivateMessage(userId, subject, message):
    MyGlobals.Logger.info("Sending private message on PTP.")

    # We need to load the send message page for the authentication key.
    result = MyGlobals.session.get(
        "https://passthepopcorn.me/inbox.php?action=compose&to=%s" % userId
    )
    response = result.text
    CheckIfLoggedInFromResponse(result, response)

    matches = re.search(
        r"""<input type="hidden" name="auth" value="(.+)" />""", response
    )
    if not matches:
        MyGlobals.Logger.info("Authorization key couldn't be found.")
        return

    auth = matches.group(1)

    # Send the message.
    # We always use HTTPS for sending message because if "Force HTTPS" is enabled in the profile then the HTTP message sending is not working.

    postData = {
        "toid": userId,
        "subject": subject,
        "body": message,
        "auth": auth,
        "action": "takecompose",
        "AntiCsrfToken": Settings.AntiCsrfToken,
    }

    result = MyGlobals.session.post("https://passthepopcorn.me/inbox.php", postData)
    response = result.text
    CheckIfLoggedInFromResponse(result, response)



import fnmatch
import logging
import os
import os.path
import re
import shutil

from pathlib import Path

from dynaconf import Dynaconf, Validator

from PtpUploader.PtpUploaderException import PtpUploaderException


logger = logging.getLogger(__name__)
config = Dynaconf(
    envvar_prefix=False,
    settings_files=[
        Path(Path(__file__).parent, "config.default.yml"),
        Path("~/.config/ptpuploader/config.yml").expanduser(),
        ".secrets.yml",
    ],
    environments=False,
    load_dotenv=True,
    validators=[Validator("uploader.srrdb_screne_check", default=False)],
)


class Settings:
    @staticmethod
    def MakeListFromExtensionString(extensions: str):
        # Make sure everything is in lower case in the settings.
        return [i.strip().lower() for i in extensions.split(",")]

    # This makes a list of TagList.
    # Eg.: "A B, C, D E" will become [ [ "A", "B" ], [ "C" ], [ "D", "E" ] ]
    @staticmethod
    def MakeListOfListsFromString(extensions: str):
        return [i.split(" ") for i in Settings.MakeListFromExtensionString(extensions)]

    @staticmethod
    def __HasValidExtensionToUpload(path, extensions):
        tempPath = path.lower()
        for extension in extensions:
            if fnmatch.fnmatch(tempPath, "*." + extension):
                return True

        return False

    @staticmethod
    def HasValidVideoExtensionToUpload(path):
        return Settings.__HasValidExtensionToUpload(
            path, Settings.VideoExtensionsToUpload
        )

    @staticmethod
    def HasValidAdditionalExtensionToUpload(path):
        return Settings.__HasValidExtensionToUpload(
            path, Settings.AdditionalExtensionsToUpload
        )

    @staticmethod
    def IsFileOnIgnoreList(path):
        path = os.path.basename(path)  # We only filter the filenames.
        path = path.lower()
        for ignoreFile in Settings.IgnoreFile:
            if re.match(ignoreFile, path) is not None:
                return True
        return False

    @staticmethod
    def GetAnnouncementWatchPath() -> Path:
        return Path(config.work_dir, "announcement")

    @staticmethod
    def GetAnnouncementInvalidPath() -> Path:
        return Path(config.work_dir, "announcement/invalid")

    @staticmethod
    def GetJobLogPath() -> Path:
        return Path(config.work_dir, "log/job")

    @staticmethod
    def GetTemporaryPath() -> Path:
        return Path(config.work_dir, "temporary")

    @staticmethod
    def __LoadSceneGroups(path):
        groups = []
        with open(path) as handle:
            for line in handle.readlines():
                groupName = line.strip().lower()
                if groupName:
                    groups.append(groupName)
        return groups

    @staticmethod
    def LoadSettings():
        # TODO: These aren't actually required for ReleaseMaker, make it so
        if not (
            config.ptp.announce_url and config.ptp.username and config.ptp.password
        ):
            raise PtpUploaderException(
                "Make sure the username, password and announce URL are set in the config!"
            )
        if not config.work_dir:
            raise PtpUploaderException(
                "Make sure the work directory is set in the config!"
            )
        Settings.VideoExtensionsToUpload = config.uploader.video_files
        Settings.AdditionalExtensionsToUpload = config.uploader.additional_files
        Settings.TorrentClient = None
        Settings.IgnoreFile = config.uploader.ignore_files
        Settings.PtpAnnounceUrl = config.ptp.announce_url
        Settings.PtpUserName = config.ptp.username
        Settings.PtpPassword = config.ptp.password

        Settings.PtpImgApiKey = config.image_host.ptpimg.api_key
        Settings.OnSuccessfulUpload = config.hook.on_upload

        Settings.FfmpegPath = config.tools.ffmpeg.path
        Settings.MediaInfoPath = config.tools.mediainfo.path
        Settings.MplayerPath = config.tools.mplayer.path
        Settings.MpvPath = config.tools.mpv.path
        Settings.UnrarPath = config.tools.unrar.path
        Settings.ImageMagickConvertPath = config.tools.imagemagick.path

        Settings.WorkingPath = config.work_dir

        Settings.AllowReleaseTag = Settings.MakeListOfListsFromString(
            config.source._default.allow_tags
        )
        Settings.IgnoreReleaseTag = Settings.MakeListOfListsFromString(
            config.source._default.ignore_tags
        )

        scene_file = Path(
            os.path.expanduser("~/.config/ptpuploader"), "scene_groups.txt"
        )
        if not scene_file.exists():
            scene_file = Path(Path(__file__).parent, "SceneGroups.txt")
        Settings.SceneReleaserGroup = Settings.__LoadSceneGroups(scene_file)

        Settings.GreasemonkeyTorrentSenderPassword = config.web.api_key
        Settings.OverrideScreenshots = config.uploader.override_screenshots
        Settings.ForceDirectorylessSingleFileTorrent = (
            config.uploader.force_directoryless_single_file
        )
        Settings.PersonalRip = config.uploader.is_personal
        Settings.ReleaseNotes = config.uploader.release_notes
        Settings.SkipDuplicateChecking = config.uploader.skip_duplicate_checking

        Settings.SizeLimitForAutoCreatedJobs = (
            float(config.source._default.max_size) * 1024 * 1024 * 1024
        )
        Settings.StopIfSynopsisIsMissing = (
            config.source._default.stop_if_synopsis_missing
        )
        Settings.StopIfCoverArtIsMissing = config.source._default.stop_if_art_missing
        Settings.StopIfImdbRatingIsLessThan = config.source._default.min_imdb_rating
        Settings.StopIfImdbVoteCountIsLessThan = config.source._default.min_imdb_votes
        Settings.MediaInfoTimeOut = config.tools.mediainfo.timeout

        Settings.TorrentClientName = config.client.use
        # Hack to let env var take precedence
        Settings.TorrentClientAddress = (
            config["client"][config.client.use.upper()]["ADDRESS"]
            or config["client"][config.client.use]["address"]
        )

        # Create required directories.
        Settings.GetAnnouncementInvalidPath().mkdir(parents=True, exist_ok=True)
        Settings.GetJobLogPath().mkdir(parents=True, exist_ok=True)
        Settings.GetTemporaryPath().mkdir(parents=True, exist_ok=True)

    @staticmethod
    def CreateClient():
        use = config.client.use.lower()
        # Hack to let env var take precedence
        address = (
            config["client"][config.client.use.upper()]["ADDRESS"]
            or config["client"][config.client.use]["address"]
        )
        if use == "transmission":
            from PtpUploader.Tool.Transmission import Transmission

            Settings.TorrentClient = Transmission(
                address.split(":")[0],
                address.split(":")[1],
            )
        else:
            from PtpUploader.Tool.Rtorrent import Rtorrent

            Settings.TorrentClient = Rtorrent(address)

    @staticmethod
    def GetTorrentClient():
        if Settings.TorrentClient is None:
            Settings.CreateClient()
        return Settings.TorrentClient

    @staticmethod
    def VerifyPaths():
        logger.info("Checking paths")

        if shutil.which(Settings.MediaInfoPath) is None:
            logger.critical(
                "Mediainfo not found with command '%s'!", Settings.MediaInfoPath
            )
            return False

        if (
            shutil.which(Settings.MpvPath) is None
            and shutil.which(Settings.MplayerPath) is None
            and shutil.which(Settings.FfmpegPath) is None
        ):
            logger.critical("At least one of mpv, mplayer or ffmpeg is required!")
            return False

        # Optional
        if Settings.UnrarPath and not shutil.which(Settings.UnrarPath):
            logger.error("Unrar path is set but not found: %s", Settings.UnrarPath)
        if Settings.ImageMagickConvertPath and not shutil.which(
            Settings.ImageMagickConvertPath
        ):
            logger.error(
                "ImageMagick path is set but not found: %s",
                Settings.ImageMagickConvertPath,
            )

        return True



from PtpUploader.MyGlobals import MyGlobals
from PtpUploader.Settings import Settings
from PtpUploader.Source.Cinemageddon import Cinemageddon
from PtpUploader.Source.File import File
from PtpUploader.Source.Karagarga import Karagarga
from PtpUploader.Source.Prowlarr import Prowlarr
from PtpUploader.Source.Torrent import Torrent


class SourceFactory:
    def __init__(self):
        self.Sources = {}

        self.__AddSource(File())
        self.__AddSource(Torrent())

        self.__AddSource(Cinemageddon())
        self.__AddSource(Karagarga())
        self.__AddSource(Prowlarr())

        MyGlobals.Logger.info("Sources initialized.")

    def __AddSource(self, source):
        if source.IsEnabled():
            source.LoadSettings(Settings)
            source.Login()
        self.Sources[source.Name] = source

    def GetSource(self, sourceName):
        # We don't want to throw KeyError exception, so we use get.
        return self.Sources.get(sourceName)

    def GetSourceAndIdByUrl(self, url):
        for _, source in list(self.Sources.items()):
            source_id = source.GetIdFromUrl(url)
            if source_id:
                return source, source_id

        return None, ""



import requests

from PtpUploader.ImageHost.Base import ImageSite
from PtpUploader.PtpUploaderException import PtpUploaderException


class PtpImg(ImageSite):
    def __init__(self, logger=None):
        self.name = "ptpimg"
        self.endpoint = "https://ptpimg.me/upload.php"
        super().__init__(logger)
        if not self.config.api_key:
            raise PtpUploaderException("ptpimg.me API key is not set")

    def upload_url(self, url: str):
        return self.upload({"link-upload": url}, {})

    def upload_path(self, path: str):
        with open(path, "rb") as imageHandle:
            return self.upload({}, {"file-upload": imageHandle})

    def upload(self, data, files):
        data["api_key"] = self.config.api_key
        response = requests.post("https://ptpimg.me/upload.php", data=data, files=files)
        response.raise_for_status()
        try:
            rjson = response.json()[0]
            return "https://ptpimg.me/{}.{}".format(rjson["code"], rjson["ext"])
        except (ValueError, KeyError):
            self.logger.exception(
                "Got an exception while loading JSON response from ptpimg.me. Response: '{}'.".format(
                    str(response.text())
                )
            )
            raise



"""
Django settings for mysite project.

Generated by 'django-admin startproject' using Django 3.2.7.

For more information on this file, see
https://docs.djangoproject.com/en/3.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/3.2/ref/settings/
"""

import os

from pathlib import Path

import dynaconf

from PtpUploader.Settings import Settings


Settings.LoadSettings()

X_FRAME_OPTIONS = "SAMEORIGIN"

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/3.2/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "django-insecure-3)felyc-ikqtmr57j)1r^86zd^p)#oc2hkmbduuo!ssj4!w^w5"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = False

ALLOWED_HOSTS = "*"

LOGIN_URL = "/admin/login/"

# Application definition

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "PtpUploader.web",
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

ROOT_URLCONF = "PtpUploader.web.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

WSGI_APPLICATION = "PtpUploader.web.wsgi.application"


# Database
# https://docs.djangoproject.com/en/3.2/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": Path(Settings.WorkingPath, "db.sqlite3"),
    },
}


# Password validation
# https://docs.djangoproject.com/en/3.2/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]


# Internationalization
# https://docs.djangoproject.com/en/3.2/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/3.2/howto/static-files/

STATIC_URL = "/static/"

# Default primary key field type
# https://docs.djangoproject.com/en/3.2/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "root_formatter": {
            "format": "%(asctime)-15s %(levelname)-5s %(name)-11s %(message)s"
        }
    },
    "handlers": {
        "console": {
            "level": "INFO",
            "class": "logging.StreamHandler",
            "formatter": "root_formatter",
        },
    },
    "loggers": {
        "": {
            "handlers": [
                "console",
            ],
            "level": "DEBUG",
            "propagate": True,
        },
        "django": {
            "handlers": ["console"],
            "level": os.getenv("DJANGO_LOG_LEVEL", "ERROR"),
            "propagate": False,
        },
    },
}


settings = dynaconf.DjangoDynaconf(
    __name__,
    SETTINGS_FILE_FOR_DYNACONF=[
        Path(Path(__file__).parent, "config.default.yml"),
        Path("~/.config/ptpuploader/config.yml").expanduser(),
        ".secrets.yaml",
    ],
    ENVIRONMENTS_FOR_DYNACONF=False,
)



import logging
import os

from pathlib import Path
from typing import List

import rarfile  # type: ignore

from unidecode import unidecode

from PtpUploader.PtpUploaderException import PtpUploaderException
from PtpUploader.Settings import config


# from PtpUploader.ReleaseInfo import ReleaseInfo

log = logging.getLogger(__name__)


def parse_directory(release_info):
    """Split the release upload path into video and non-video files"""
    if release_info.SourceIsAFile():
        return [release_info.GetReleaseDownloadPath()], []
    path = Path(release_info.GetReleaseUploadPath())
    return find_allowed_files(path)


def find_allowed_files(path: Path):
    video_files = []
    addtl_files = []
    for child in Path(path).rglob("*"):
        if child.is_file():
            if child.suffix.lower().strip(".") in config.uploader.video_files:
                video_files.append(child)
            elif child.suffix.lower().strip(".") in config.uploader.additional_files:
                addtl_files.append(child)
    return video_files, addtl_files


def extract_release(release_info):
    """This function essentially just configures all the variables to be passed to extract_files,
    purely to provide separation of concerns"""
    ignored_top_dirs: List[str] = []
    allow_exts: List[str] = (
        config.uploader.video_files + config.uploader.additional_files
    )
    source = Path(release_info.GetReleaseDownloadPath())
    if release_info.SourceIsAFile():
        if source.suffix.lower().strip(".") in config.uploader.video_files:
            log.info("Source is a single file, skipping extraction")
            return
        else:
            raise PtpUploaderException(
                f"Single file {source} is not a known video extension ({config.uploader.video_files})"
            )

    dest = Path(release_info.GetReleaseUploadPath())
    handle_scene_folders = False
    # Allow an existing directory only if it's empty
    if dest.exists() and list(dest.iterdir()):
        raise PtpUploaderException(
            f"Can't make destination directory '{dest}' because path exists and is not empty."
        )
    if not source.exists():
        raise PtpUploaderException(f"Source '{source}' does not exist.")
    if release_info.IsSceneRelease():
        handle_scene_folders = True
    # Blu-rays can contain funky files, just allow all
    if release_info.IsBlurayImage():
        allow_exts = ["*"]
        handle_scene_folders = False
    # Fix DVDs without top level VIDEO_TS
    if release_info.IsDvdImage() and "VIDEO_TS" not in [
        c.name for c in source.glob("*")
    ]:
        dest = Path(dest, "VIDEO_TS")
    if (
        release_info.AnnouncementSource.Name == "file"
        and not release_info.SourceIsAFile()
    ):
        ignored_top_dirs = ["PTP"]
    log.info("Extracting release from '%s' to '%s'", source, dest)
    try:
        extract_files(source, dest, allow_exts, ignored_top_dirs, handle_scene_folders)
    except Exception:
        # Clean up the directory if it's empty
        try:
            dest.rmdir()
        except OSError:
            pass
        raise


def extract_files(
    source: Path,
    dest: Path,
    allow_exts: List[str],
    ignored_top_dirs: List[str],
    handle_scene_folders: bool = False,
    dry_run: bool = False,  # Exists for testing purposes
):
    """This is the method to actually extract files. That usually
    means just hardlinking any allowed files into the same tree
    structure, but there is some logic to handle things like
    RARs. Importantly, though, this function has no concept of what
    the release object looks like. This helps to separate out the
    'business logic'.

    """
    if source.is_file() and (
        source.suffix.lower().strip(".") in allow_exts or allow_exts == ["*"]
    ):
        dest = Path(dest, unidecode(str(Path(source.name))))
        if dry_run:
            print(f"{source} -> {dest}")
        else:
            os.link(source, dest)
    for child in source.rglob("*"):
        if not child.is_file():
            continue
        if len(child.parent.parts) == 0:
            top_dir = "."
        else:
            top_dir = child.relative_to(source).parts[0]
        if top_dir in ignored_top_dirs:
            continue
        child_dest = Path(dest, unidecode(str(child.relative_to(source))))
        # Move scene subtitles to the top level
        if handle_scene_folders:
            if top_dir.lower().startswith("cd") or top_dir in [
                "sub",
                "subs",
                "subtitle",
                "subtitles",
            ]:
                child_dest = Path(dest, child.name)
        # Extract any RARs
        if child.suffix == ".rar":
            try:
                rar = rarfile.RarFile(child)
                rar.infolist()
            except Exception as e:
                log.error("Cannot unrar file %s: %s", child, e)
                continue
            for f in rar.infolist():
                if f.is_file and (
                    Path(f.filename).suffix.lower().strip(".") in allow_exts
                    or "*" in allow_exts
                ):
                    child_dest = Path(dest, unidecode(f.filename))
                    log.info(f"unrar {f.filename} from {child} -> {child_dest}")
                    if not dry_run:
                        dest.mkdir(parents=True, exist_ok=True)
                        with child_dest.open("wb") as fh:
                            fh.write(rar.open(f).readall())
        # Or just hard link
        elif child.suffix.lower().strip(".") in allow_exts or "*" in allow_exts:
            if dry_run:
                print(f"{child} -> {child_dest}")
            else:
                child_dest.parent.mkdir(parents=True, exist_ok=True)
                os.link(child, child_dest)




import argparse
import re
import os
from typing import Optional
from pathlib import Path

import django
import requests
from pyrosimple.util import metafile

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "PtpUploader.web.settings")
django.setup()

from PtpUploader import release_extractor, Ptp, MyGlobals
from PtpUploader.MyGlobals import MyGlobals
from PtpUploader.PtpUploaderException import PtpUploaderException
from PtpUploader.ReleaseDescriptionFormatter import ReleaseDescriptionFormatter
from PtpUploader.ReleaseInfo import ReleaseInfo
from PtpUploader.Settings import Settings
from PtpUploader.Tool import Mktor


class ReleaseInfoMaker:
    def __init__(self, path):
        self.Path = path
        self.ReleaseName = None
        self.WorkingDirectory = None
        self.TorrentDataPath = None
        self.VideoFiles = []
        self.AdditionalFiles = []

    def CollectVideoFiles(self):
        self.Path = os.path.abspath(self.Path)

        if os.path.isdir(self.Path):
            # Make sure that path doesn't ends with a trailing slash or else os.path.split would return with wrong values.
            self.Path = self.Path.rstrip("\\/")

            (
                self.VideoFiles,
                self.AdditionalFiles,
            ) = release_extractor.find_allowed_files(
                self.Path,
            )
            if len(self.VideoFiles) <= 0:
                print("Path '%s' doesn't contain any videos!" % self.Path)
                return False

            # We use the parent directory of the path as the working directory.
            # Release name will be the directory's name. Eg. it will be "anything" for "/something/anything"
            self.WorkingDirectory, self.ReleaseName = os.path.split(self.Path)
            self.TorrentDataPath = self.Path
        elif os.path.isfile(self.Path):
            self.VideoFiles.append(self.Path)

            # We use same the directory where the file is as the working directory.
            # Release name will be the file's name without extension.
            self.WorkingDirectory, self.ReleaseName = os.path.split(self.Path)
            self.ReleaseName, _ = os.path.splitext(self.ReleaseName)
            self.TorrentDataPath = self.WorkingDirectory
        else:
            print("Path '%s' doesn't exist!" % self.Path)
            return False

        return True

    def MarkAsDvdImageIfNeeded(self, releaseInfo):
        for file in self.AdditionalFiles:
            if str(file).lower().endswith(".ifo"):
                releaseInfo.Codec = "DVD5"
                # Make sure that ReleaseDescriptionFormatter will recognize this as a DVD image.
                if not releaseInfo.IsDvdImage():
                    raise PtpUploaderException(
                        "Codec is set to DVD5, yet release info says that this is not a DVD image."
                    )

                return

    def MarkAsBlurayImageIfNeeded(self, releaseInfo):
        if os.path.isdir(self.Path) and "BDMV" in os.listdir(self.Path):
            releaseInfo.Codec = "BD25"

    def SaveReleaseDescriptionFile(
        self, logger, releaseDescriptionFilePath, createScreens
    ):
        releaseInfo = ReleaseInfo()
        releaseInfo.Logger = logger
        releaseInfo.ReleaseName = self.ReleaseName
        releaseInfo.ReleaseUploadPath = self.TorrentDataPath
        self.MarkAsDvdImageIfNeeded(releaseInfo)
        self.MarkAsBlurayImageIfNeeded(releaseInfo)

        outputImageDirectory = self.WorkingDirectory
        releaseDescriptionFormatter = ReleaseDescriptionFormatter(
            releaseInfo,
            self.VideoFiles,
            self.AdditionalFiles,
            outputImageDirectory,
            createScreens,
        )
        releaseDescription = releaseDescriptionFormatter.Format(includeReleaseName=True)

        logger.info("Saving to description file %r", releaseDescriptionFilePath)
        with open(releaseDescriptionFilePath, "w") as handle:
            handle.write(releaseDescription)

    def MakeReleaseInfo(
        self,
        args,
        createTorrent=True,
        createScreens=True,
        setDescription: Optional[str] = None,
    ):
        logger = MyGlobals.Logger

        if not self.CollectVideoFiles():
            return

        # Make sure the files we are generating are not present.

        releaseDescriptionFilePath = os.path.join(
            self.WorkingDirectory,
            "PTP " + self.ReleaseName + ".release description.txt",
        )
        if os.path.exists(releaseDescriptionFilePath) and not args.force:
            print(
                "Can't create release description because '%s' already exists!"
                % releaseDescriptionFilePath
            )
            return

        torrentName = "PTP " + self.ReleaseName + ".torrent"
        torrentPath = os.path.join(self.WorkingDirectory, torrentName)
        if createTorrent and os.path.exists(torrentPath):
            print("Can't create torrent because '%s' already exists!" % torrentPath)
            return

        # Save the release description.
        self.SaveReleaseDescriptionFile(
            logger, releaseDescriptionFilePath, createScreens
        )

        if setDescription is not None:
            tID = None
            if os.path.exists(setDescription):
                meta = metafile.Metafile.from_file(Path(setDescription))
                setDescription = meta["comment"]
            if setDescription and "torrentid=" in setDescription:
                tID = re.search("torrentid=(\d+)", setDescription).group(1)
            Ptp.Login()
            logger.info("Uploading description as a report to %s", tID)
            if not Settings.AntiCsrfToken:
                raise PtpUploaderException("No AntiCsrfToken found")
            with open(releaseDescriptionFilePath, "r") as fh:
                r = MyGlobals.session.post(
                    "https://passthepopcorn.me/reportsv2.php?action=takereport",
                    data={
                        "extra": fh.read(),
                        "torrentid": tID,
                        "categoryid": "1",
                        "type": "replacement description",
                        "submit": "true",
                        "AntiCsrfToken": Settings.AntiCsrfToken,
                    },
                )
                r.raise_for_status()

        # Create the torrent
        if createTorrent:
            Mktor.Make(logger, self.Path, torrentPath)
            MyGlobals.GetTorrentClient().AddTorrentSkipHashCheck(
                logger, torrentPath, self.TorrentDataPath
            )


def run():
    parser = argparse.ArgumentParser(
        description="PtpUploader Release Description Maker by TnS"
    )

    parser.add_argument(
        "--notorrent", action="store_true", help="skip creating and seeding the torrent"
    )
    parser.add_argument(
        "--force",
        action="store_true",
        help="overwrite the description file if it already exists",
    )
    parser.add_argument(
        "--noscreens",
        action="store_true",
        help="skip creating and uploading screenshots",
    )
    # Hidden stub option to upload description directly to PTP
    parser.add_argument("--set-description", help=argparse.SUPPRESS, default=None)
    parser.add_argument("path", nargs=1, help="The file or directory to use")

    args = parser.parse_args()

    Settings.LoadSettings()
    MyGlobals.InitializeGlobals(Settings.WorkingPath)

    releaseInfoMaker = ReleaseInfoMaker(args.path[0])
    releaseInfoMaker.MakeReleaseInfo(
        args=args,
        createTorrent=(not args.notorrent),
        createScreens=(not args.noscreens),
        setDescription=args.set_description,
    )


if __name__ == "__main__":
    run()



import logging
import os
import threading
import traceback

from PtpUploader.Job.JobRunningState import JobRunningState
from PtpUploader.PtpUploaderException import *
from PtpUploader.ReleaseInfo import ReleaseInfo
from PtpUploader.Settings import Settings


logger = logging.getLogger(__name__)


class WorkLogFilter(logging.Filter):
    def __init__(self, release_id, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.release_id = release_id

    def filter(self, record):
        if "release_id" in record.__dict__:
            return self.release_id == record.release_id
        return False


class WorkerBase:
    def __init__(self, release_id: int, stop_requested: threading.Event):
        self.Phases = []
        self.stop_requested: threading.Event = stop_requested
        self.ReleaseInfo = ReleaseInfo.objects.get(Id=release_id)
        self.logHandler = (
            self.start_worker_logging()
        )  # needed to clean up after job is finished
        self.logger = self.ReleaseInfo.logger(logger)  # Just a nice shortcut

    def __WorkInternal(self):
        if not self.Phases:
            raise NotImplementedError("Add phases to this worker")
        for phase in self.Phases:
            if self.stop_requested.is_set():
                self.ReleaseInfo.JobRunningState = JobRunningState.Paused
                self.ReleaseInfo.save()
                break

            phase()
        self.stop_worker_logging()

    def stop_worker_logging(self):
        logging.getLogger().removeHandler(self.logHandler)

    def start_worker_logging(self):
        """
        Add a log handler to separate file for current thread
        """
        path = os.path.join(Settings.GetJobLogPath(), str(self.ReleaseInfo.Id))
        log_handler = logging.FileHandler(path)

        log_handler.setLevel(logging.DEBUG)

        formatter = logging.Formatter(
            "%(asctime)-15s" " %(name)-11s" " %(levelname)-5s" " %(message)s"
        )
        log_handler.setFormatter(formatter)

        log_filter = WorkLogFilter(self.ReleaseInfo.Id)
        log_handler.addFilter(log_filter)

        log = logging.getLogger()
        log.addHandler(log_handler)
        return log_handler

    def Work(self):
        try:
            self.__WorkInternal()
            self.ReleaseInfo.clean()
        except Exception as e:
            if hasattr(e, "JobRunningState"):
                self.ReleaseInfo.JobRunningState = e.JobRunningState
            else:
                self.ReleaseInfo.JobRunningState = JobRunningState.Failed

            self.ReleaseInfo.ErrorMessage = str(e)
            self.ReleaseInfo.save()

            if isinstance(e, PtpUploaderException) and str(e).startswith("Stopping "):
                self.ReleaseInfo.logger().info(f"Received stop: {e}")
            else:
                self.ReleaseInfo.logger().exception(traceback.format_exc())
            raise



import datetime
import logging
import os
import pickle
import sys

from pathlib import Path

import requests


class MyGlobalsClass:
    def __init__(self):
        self.Logger = None
        self.PtpUploader = None
        self.SourceFactory = None
        self.TorrentClient = None

        self.session = requests.session()
        self.session.headers.update(
            {
                "User-Agent": "Mozilla/5.0 (Windows NT 6.3; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/40.0.2214.45 Safari/537.36"
            }
        )

        # Use cloudflare-scrape if installed.
        try:
            from cfscrape import CloudflareAdapter  # type: ignore

            self.session.mount("https://", CloudflareAdapter())
        except ImportError:
            pass

    def InitializeGlobals(self, workingPath):
        from PtpUploader.Settings import config

        self.InitializeLogger(workingPath)
        self.cookie_file: Path = Path(workingPath).joinpath("cookies.pickle")
        self.cookie_file: Path = Path(config.cookie_file).expanduser()
        if self.cookie_file.exists() and self.cookie_file.is_file():
            with self.cookie_file.open("rb") as fh:
                self.session.cookies = pickle.load(fh)

    def SaveCookies(self):
        with self.cookie_file.open("wb") as fh:
            pickle.dump(self.session.cookies, fh)

    # workingPath from Settings.WorkingPath.
    def InitializeLogger(self, workingPath):
        # This will create the log directory too.
        self.Logger = logging.getLogger(__name__)

    # Inline imports are used here to avoid unnecessary dependencies.
    def GetTorrentClient(self):
        if self.TorrentClient is None:
            from PtpUploader.Settings import Settings

            if Settings.TorrentClientName.lower() == "transmission":
                from PtpUploader.Tool.Transmission import Transmission

                self.TorrentClient = Transmission(
                    Settings.TorrentClientAddress, Settings.TorrentClientPort
                )
            else:
                from PtpUploader.Tool.Rtorrent import Rtorrent

                self.TorrentClient = Rtorrent(Settings.TorrentClientAddress)
        return self.TorrentClient


MyGlobals = MyGlobalsClass()



from django.forms import (
    BooleanField,
    CharField,
    ChoiceField,
    ClearableFileInput,
    CheckboxInput,
    FileField,
    Form,
    HiddenInput,
    IntegerField,
    ModelForm,
    MultipleChoiceField,
    PasswordInput,
    Textarea,
    TextInput,
)

from PtpUploader import ImageHost, ptp_subtitle
from PtpUploader.Job.JobStartMode import JobStartMode
from PtpUploader.ReleaseInfo import ReleaseInfo
from PtpUploader.Settings import config


class SettingsForm(Form):
    image_host_use = ChoiceField(
        choices=[(x, x) for x in ImageHost.list_hosts()],
        initial=config.image_host.use,
        label="Host",
    )
    ptp_username = CharField(initial=config.ptp.username, label="Username")
    ptp_password = CharField(
        initial=config.ptp.password,
        label="Password",
        widget=TextInput(attrs={"type": "password"}),
    )
    ptp_announce_url = CharField(initial=config.ptp.announce_url, label="Announce URL")
    client_use = ChoiceField(
        choices=[(x, x) for x in ["rtorrent", "transmission"]],
        initial=config.client.use,
        label="Use",
    )
    client_address = CharField(
        initial=config.client[config.client.use].address,
        label="Address",
        help_text="Automatically determined if not set",
        required=False,
    )


class BulkReleaseForm(Form):
    Links = CharField(widget=Textarea(attrs={"placeholder": "Links"}), required=False)
    Paths = CharField(widget=Textarea(attrs={"placeholder": "Paths"}), required=False)
    Files = FileField(
        widget=ClearableFileInput(
            attrs={
                "class": "file-input",
                "multiple": "multiple",
                "accept": "application/x-bittorrent",
            }
        ),
        required=False,
    )


class ReleaseForm(ModelForm):
    Type = ChoiceField(choices=ReleaseInfo.TypeChoices.choices, required=False)
    Codec = ChoiceField(choices=ReleaseInfo.CodecChoices.choices, required=False)
    Container = ChoiceField(
        choices=ReleaseInfo.ContainerChoices.choices, required=False
    )
    Source = ChoiceField(choices=ReleaseInfo.SourceChoices.choices, required=False)
    SceneRelease = BooleanField(
        initial=config.uploader.is_scene,
        widget=CheckboxInput(attrs={"checked": config.uploader.is_scene}),
        required=False,
    )
    ResolutionType = ChoiceField(
        choices=ReleaseInfo.ResolutionChoices.choices, required=False
    )
    Subtitles = MultipleChoiceField(
        choices=[(str(k), v[0].title()) for k, v in ptp_subtitle.subtitle_ids.items()],
        required=False,
    )
    if config.uploader.skip_duplicate_checking:
        init = 0
    else:
        init = None
    DuplicateCheckCanIgnore = IntegerField(
        initial=init,
        widget=HiddenInput(),
    )
    Tags = MultipleChoiceField(
        required=False,
        choices=[
            (v, v)
            for v in (
                "action",
                "adventure",
                "animation",
                "arthouse",
                "asian",
                "biography",
                "camp",
                "comedy",
                "crime",
                "cult",
                "documentary",
                "drama",
                "experimental",
                "exploitation",
                "family",
                "fantasy",
                "film.noir",
                "history",
                "horror",
                "martial.arts",
                "musical",
                "mystery",
                "performance",
                "philosophy",
                "politics",
                "romance",
                "sci.fi",
                "short",
                "silent",
                "sport",
                "thriller",
                "video.art",
                "war",
                "western",
            )
        ],
    )
    # Fields that don't map the release object
    ForceUpload = BooleanField(required=False, initial=False)
    TrumpableNoEnglish = BooleanField(required=False, initial=False)
    TrumpableHardSubs = BooleanField(required=False, initial=False)
    TorrentLink = CharField(required=False, widget=TextInput(attrs={"size": "60"}))
    LocalFile = CharField(required=False, widget=TextInput(attrs={"size": "60"}))
    JobStartMode = CharField(required=False)
    RawFile = FileField(
        required=False,
        widget=ClearableFileInput(attrs={"accept": "application/x-bittorrent"}),
    )

    class Meta:
        model = ReleaseInfo
        exclude = [
            "JobRunningState",
            "ScheduleTime",
            "FinishedJobPhase",
            "Size",
        ]
        widgets = {
            "ImdbId": TextInput(attrs={"size": "60"}),
            "Directors": TextInput(attrs={"size": "60"}),
            "YouTubeId": TextInput(attrs={"size": "60"}),
            "CoverArtUrl": TextInput(attrs={"size": "60"}),
            "Title": TextInput(attrs={"size": "53"}),
            "Year": TextInput(attrs={"size": "4", "placeholder": "Year"}),
            "RemasterTitle": TextInput(attrs={"size": "53"}),
            "RemasterYear": TextInput(attrs={"size": "4", "placeholder": "Year"}),
            "MovieDescription": Textarea(attrs={"cols": "60", "rows": "8"}),
            "ReleaseNotes": Textarea(attrs={"cols": "60", "rows": "8"}),
            "ReleaseName": TextInput(attrs={"size": "60"}),
            "DuplicateCheckCanIgnore": HiddenInput(),
            "SourceOther": TextInput(attrs={"size": "5"}),
            "Resolution": TextInput(attrs={"size": "8"}),
            "ContainerOther": TextInput(attrs={"size": "5"}),
            "CodecOther": TextInput(attrs={"size": "5"}),
        }

    def clean(self):
        data = super().clean()
        data["Tags"] = ",".join(data["Tags"])
        data["JobStartMode"] = JobStartMode.Manual
        if data["ForceUpload"]:
            data["JobStartMode"] = JobStartMode.ManualForced
        del data["ForceUpload"]
        data["Trumpable"] = []
        if data["TrumpableNoEnglish"]:
            data["Trumpable"] += [14]
        if data["TrumpableHardSubs"]:
            data["Trumpable"] += [4]
        return data



import os
import re
import subprocess

from pathlib import Path

from PtpUploader.PtpUploaderException import PtpUploaderException
from PtpUploader.Settings import Settings
from PtpUploader.Tool.MediaInfo import MediaInfo


class Ffmpeg:
    def __init__(self, logger, inputVideoPath):
        self.Logger = logger
        self.InputVideoPath = inputVideoPath
        self.ScaleSize = None

        self.__CalculateSizeAccordingToAspectRatio()

    def __CalculateSizeAccordingToAspectRatio(self):
        # Get resolution and pixel aspect ratio from FFmpeg.
        args = [Settings.FfmpegPath, "-i", self.InputVideoPath]
        proc: subprocess.CompletedProcess = subprocess.run(args, capture_output=True)
        result: str = proc.stderr.decode("utf-8", "ignore")

        # Formatting can be one of the following. PAR can be SAR too.
        # Stream #0.0(eng): Video: h264, yuv420p, 1280x544, PAR 1:1 DAR 40:17, 24 tbr, 1k tbn, 48 tbc
        # Stream #0.0[0x1e0]: Video: mpeg2video, yuv420p, 720x480 [PAR 8:9 DAR 4:3], 7500 kb/s, 29.97 tbr, 90k tbn, 59.94 tbc
        match = re.search(r"(\d+)x(\d+), [SP]AR \d+:\d+ DAR (\d+):(\d+)", result)
        if match is None:
            match = re.search(r"(\d+)x(\d+) \[[SP]AR \d+:\d+ DAR (\d+):(\d+)", result)
        if match is None:
            return

        width: int = int(match.group(1))
        height: int = int(match.group(2))
        darX: int = int(match.group(3))
        darY: int = int(match.group(4))
        # We ignore invalid resolutions, invalid aspect ratios and aspect ratio 1:1.
        if (
            width <= 0
            or height <= 0
            or darX <= 0
            or darY <= 0
            or (darX == 1 and darY == 1)
        ):
            return

        # If we are a DVD, sometimes the DAR can get messed up between the VOB and IFO
        # We'll use the mediainfo to draw our conclusions from instead
        ifoPath = Path(self.InputVideoPath[:-5] + "0.IFO")
        if ifoPath.exists():
            self.Logger.debug("Fetching DAR information from '%s'", ifoPath)
            m = MediaInfo(self.Logger, ifoPath, "")
            try:
                darX = float(m.DAR)
                darY = 1
            except ValueError:
                try:
                    darX = int(m.DAR.split(":")[0].strip())
                    darY = int(m.DAR.split(":")[1].strip())
                except (ValueError, IndexError):
                    pass

        # Choose whether we resize height or width.
        if (float(darX) / darY) >= (float(width) / height):
            # Resize width
            newWidth = (height * darX) / darY
            newWidth = int(newWidth)
            if abs(newWidth - width) <= 1:
                return

            # For FFmpeg frame size must be a multiple of 2.
            if (newWidth % 2) != 0:
                newWidth += 1

            self.ScaleSize = f"{newWidth}x{height}"
        else:
            # Resize height
            newHeight = (width * darY) / darX
            newHeight = int(newHeight)
            if abs(newHeight - height) <= 1:
                return

            # For FFmpeg frame size must be a multiple of 2.
            if (newHeight % 2) != 0:
                newHeight += 1

            self.ScaleSize = f"{width}x{newHeight}"

    def MakeScreenshotInPng(self, timeInSeconds, outputPngPath):
        self.Logger.info(
            "Making screenshot with ffmpeg from '%s' to '%s'."
            % (self.InputVideoPath, outputPngPath)
        )

        # -an: disable audio
        # -sn: disable subtitle
        # There is no way to set PNG compression level. :(
        args = []
        time = str(int(timeInSeconds))
        args = [
            Settings.FfmpegPath,
            "-an",
            "-sn",
            "-ss",
            time,
            "-i",
            self.InputVideoPath,
            "-vcodec",
            "png",
            "-vframes",
            "1",
            "-pix_fmt",
            "rgb24",
        ]
        if self.ScaleSize is not None:
            self.Logger.info(
                "Pixel aspect ratio wasn't 1:1, scaling video to resolution: '%s'."
                % self.ScaleSize
            )
            args += [
                "-s",
                self.ScaleSize,
            ]
        args += ["-y", outputPngPath]
        errorCode = subprocess.call(args)
        if errorCode != 0:
            raise PtpUploaderException(
                "Process execution '%s' returned with error code '%s'."
                % (args, errorCode)
            )



import re
import subprocess

from pathlib import Path

from PtpUploader.Helper import GetSizeFromText
from PtpUploader.PtpUploaderException import PtpUploaderException
from PtpUploader.Settings import Settings


class MediaInfoException(PtpUploaderException):
    pass


class MediaInfo:
    # removePathFromCompleteName: this part will be removed from the path listed at "Complete Name". If removePathFromCompleteName is empty then it will be left as it is.
    def __init__(self, logger, path, removePathFromCompleteName):
        self.Path = path
        self.RemovePathFromCompleteName = removePathFromCompleteName
        self.FormattedMediaInfo = ""
        self.DurationInSec = 0
        self.Container = ""
        self.Codec = ""
        # Mediainfo can possibly return a larger file size than
        # the one actually being read. RealFileSize reflects the
        # size on disk, while FileSize is parsed from the output.
        self.FileSize: int = 0
        self.RealFileSize: int = 0
        self.Width: int = 0
        self.Height: int = 0
        self.DAR: str = ""
        self.VideoWritingLibrary = ""
        self.Subtitles = []

        self.MediaInfoArgs = [Settings.MediaInfoPath, self.Path]

        self.__ParseMediaInfo(logger)
        self.__ValidateParsedMediaInfo()

    # Returns with the output of MediaInfo.
    def __ReadMediaInfo(self, logger):
        logger.info("Reading media info from '%s'." % self.Path)

        # MediaInfo is buggy on some videos and takes a lot of time to finish. We limit this time to the user specified maximum.
        proc = subprocess.run(
            self.MediaInfoArgs,
            timeout=Settings.MediaInfoTimeOut,
            capture_output=True,
            check=True,
        )
        self.RealFileSize = Path(self.Path).stat().st_size
        return proc.stdout.decode("utf-8", "ignore")

    # removePathFromCompleteName: see MediaInfo's constructor
    # Returns with the media infos for all files in videoFiles.
    @staticmethod
    def ReadAndParseMediaInfos(logger, videoFiles, removePathFromCompleteName):
        mediaInfos = []
        for video in videoFiles:
            mediaInfo = MediaInfo(logger, video, removePathFromCompleteName)
            mediaInfos.append(mediaInfo)

        return mediaInfos

    @staticmethod
    def __ParseSize(mediaPropertyValue):
        mediaPropertyValue = mediaPropertyValue.replace("pixels", "")
        mediaPropertyValue = mediaPropertyValue.replace(" ", "")
        # Resolution may contain space, so remove. Eg.: 1 280
        return int(mediaPropertyValue)

    # Matches duration in the following format. All units and spaces are optional.
    # 1h 2min 3s
    # 1h2mn3s
    # 900 ms
    @staticmethod
    def __GetDurationInSec(duration):
        # Nice regular expression. :)
        # r means to do not unescape the string
        # ?: means to do not store that group capture
        duration = duration.replace("mn", "min")
        duration = duration.replace(" ", "")
        match = re.match(r"(?:(\d+)h)?(?:(\d+)min)?(?:(\d+)s)?", duration)
        if not match:
            return 0

        # It's returned milliseconds, but we're dealing in seconds
        if duration.endswith("ms"):
            duration = 1
        else:
            duration = 0
        if match.group(1):
            duration += int(match.group(1)) * 60 * 60
        if match.group(2):
            duration += int(match.group(2)) * 60
        if match.group(3):
            duration += int(match.group(3))

        return duration

    def __MakeCompleteNameRelative(self, path):
        if len(self.RemovePathFromCompleteName) > 0:
            removePathFromCompleteName = self.RemovePathFromCompleteName.replace(
                "\\", "/"
            )
            if not removePathFromCompleteName.endswith("/"):
                removePathFromCompleteName += "/"

            path = path.replace("\\", "/")
            path = path.replace(removePathFromCompleteName, "")

        return path

    def __ParseMediaInfo(self, logger):
        # We can't simply store mediaInfoText in self.FormattedMediaInfo because the "Complete name" property gets modified.
        # (By removing the full path if needed.)
        mediaInfoText = self.__ReadMediaInfo(logger)

        section = ""
        for line in mediaInfoText.splitlines():
            if line.find(":") == -1:
                if len(line) > 0:
                    section = line
            else:
                mediaPropertyName, separator, mediaPropertyValue = line.partition(": ")
                originalMediaPropertyName = mediaPropertyName
                mediaPropertyName = mediaPropertyName.strip()

                if section == "General":
                    if mediaPropertyName == "Complete name":
                        line = (
                            originalMediaPropertyName
                            + separator
                            + self.__MakeCompleteNameRelative(mediaPropertyValue)
                        )
                    elif mediaPropertyName == "Format":
                        self.Container = mediaPropertyValue.lower()
                    elif mediaPropertyName == "Duration":
                        self.DurationInSec = MediaInfo.__GetDurationInSec(
                            mediaPropertyValue
                        )
                    elif mediaPropertyName == "File size":
                        self.FileSize = GetSizeFromText(mediaPropertyValue)
                elif section == "Video":
                    if mediaPropertyName == "Codec ID":
                        self.Codec = mediaPropertyValue.lower()
                    elif mediaPropertyName == "Width":
                        self.Width = MediaInfo.__ParseSize(mediaPropertyValue)
                    elif mediaPropertyName == "Height":
                        self.Height = MediaInfo.__ParseSize(mediaPropertyValue)
                    elif mediaPropertyName == "Writing library":
                        self.VideoWritingLibrary = mediaPropertyValue.lower()
                    elif mediaPropertyName == "Display aspect ratio":
                        self.DAR = mediaPropertyValue.lower().strip()
                elif section.startswith("Text #") or section == "Text":
                    if mediaPropertyName == "Language":
                        self.Subtitles.append(mediaPropertyValue)

            self.FormattedMediaInfo += line + "\n"

        self.FormattedMediaInfo = self.FormattedMediaInfo.strip()

    def __ValidateParsedMediaInfo(self):
        if len(self.Container) <= 0:
            raise MediaInfoException("MediaInfo returned with no container.")

        # IFOs and VOBs don't have codec.
        if len(self.Codec) <= 0 and (not self.IsIfo()) and (not self.IsVob()):
            raise MediaInfoException("MediaInfo returned with no codec.")

        # IFOs may have zero duration.
        if self.DurationInSec <= 0 and (not self.IsIfo()):
            raise MediaInfoException(
                "MediaInfo returned with invalid duration: '%s'." % self.DurationInSec
            )

        if self.Width <= 0:
            raise MediaInfoException(
                "MediaInfo returned with invalid width: '%s'." % self.Width
            )

        if self.Height <= 0:
            raise MediaInfoException(
                "MediaInfo returned with invalid height: '%s'." % self.Height
            )

    def IsAvi(self):
        return self.Container == "avi"

    def IsIfo(self):
        return self.Container == "dvd video"

    def IsMkv(self):
        return self.Container == "matroska"

    def IsMp4(self):
        return self.Container == "mpeg-4"

    def IsVob(self):
        return self.Container == "mpeg-ps"

    def IsDivx(self):
        return self.Codec in ["divx", "dx50", "div3"]

    def IsXvid(self):
        return self.Codec == "xvid"

    def IsX264(self):
        return self.Codec == "x264" or (
            (self.Codec in ["v_mpeg4/iso/avc", "avc1", "h264"])
            and self.VideoWritingLibrary.find("x264 core") == 0
        )

    def IsH264(self):
        return (
            self.Codec in ["v_mpeg4/iso/avc", "avc1", "h264"]
        ) and self.VideoWritingLibrary.find("x264 core") == -1

    def IsVc1(self):
        return self.Codec in ["wvc1", "v_ms/vfw/fourcc / wvc1"]

    def IsMpeg2Codec(self):
        return self.Codec == "v_mpeg2"



# Generated by Django 3.2.8 on 2021-11-04 03:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('web', '0012_alter_releaseinfo_screenshots'),
    ]

    operations = [
        migrations.AlterField(
            model_name='releaseinfo',
            name='Size',
            field=models.BigIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='releaseinfo',
            name='Type',
            field=models.TextField(blank=True, choices=[('Feature Film', 'Feature Film'), ('Short Film', 'Short Film'), ('Miniseries', 'Miniseries'), ('Stand-up Comedy', 'Stand-up Comedy'), ('Concert', 'Concert'), ('Live Performance', 'Live Performance'), ('Movie Collection', 'Movie Collection')], default='Feature Film'),
        ),
    ]



import subprocess

from PtpUploader.PtpUploaderException import PtpUploaderException
from PtpUploader.Settings import Settings


class Mpv:
    def __init__(self, logger, inputVideoPath):
        self.Logger = logger
        self.InputVideoPath = inputVideoPath
        self.ScaleSize = None

    def MakeScreenshotInPng(self, timeInSeconds, outputPngPath):
        self.Logger.info(
            "Making screenshot with mpv from '%s' to '%s'."
            % (self.InputVideoPath, outputPngPath)
        )

        args = [
            Settings.MpvPath,
            "--no-config",
            "--no-audio",
            "--no-sub",
            "--start=" + str(int(timeInSeconds)),
            "--frames=1",
            "--screenshot-format=png",
            "--screenshot-png-compression=9",  # doesn't seem to be working
            "--vf=lavfi=[scale='max(iw,iw*sar)':'max(ih/sar,ih)']",
            "--o=" + outputPngPath,
            self.InputVideoPath,
        ]

        errorCode = subprocess.call(args)
        if errorCode != 0:
            raise PtpUploaderException(
                "Process execution '%s' returned with error code '%s'."
                % (args, errorCode)
            )



import contextlib
import functools
import os
import shutil
import tempfile

from PtpUploader import ImageHost
from PtpUploader.PtpUploaderException import PtpUploaderException
from PtpUploader.Settings import Settings, config
from PtpUploader.Tool import Oxipng
from PtpUploader.Tool.Ffmpeg import Ffmpeg
from PtpUploader.Tool.ImageMagick import ImageMagick
from PtpUploader.Tool.Mplayer import Mplayer
from PtpUploader.Tool.Mpv import Mpv
from PtpUploader.Tool.LibMpv import LibMpv


# Blatantly stolen from https://stackoverflow.com/a/57701186
@contextlib.contextmanager
def temporary_filename(suffix=None):
    """Context that introduces a temporary file.

    Creates a temporary file, yields its name, and upon context exit, deletes it.
    (In contrast, tempfile.NamedTemporaryFile() provides a 'file' object and
    deletes the file as soon as that file object is closed, so the temporary file
    cannot be safely re-opened by another library or process.)

    Args:
      suffix: desired filename extension (e.g. '.mp4').

    Yields:
      The name of the temporary file.
    """
    try:
        f = tempfile.NamedTemporaryFile(suffix=suffix, delete=False)
        tmp_name = f.name
        f.close()
        yield tmp_name
    finally:
        os.unlink(tmp_name)


class ScreenshotMaker:
    def __init__(self, logger, inputVideoPath):
        self.Logger = logger

        self.InternalScreenshotMaker = None

        if config.tools.screenshot_tool == "mpv":
            self.InternalScreenshotMaker = Mpv(logger, inputVideoPath)
        elif config.tools.screenshot_tool == "libmpv":
            self.InternalScreenshotMaker = LibMpv(logger, inputVideoPath)
        elif config.tools.screenshot_tool == "ffmpeg":
            self.InternalScreenshotMaker = Ffmpeg(logger, inputVideoPath)
        elif config.tools.screenshot_tool == "mplayer":
            self.InternalScreenshotMaker = Mplayer(logger, inputVideoPath)
        else:
            if Settings.MpvPath and shutil.which(Settings.MpvPath):
                self.InternalScreenshotMaker = Mpv(logger, inputVideoPath)
            elif Settings.FfmpegPath and shutil.which(Settings.FfmpegPath):
                self.InternalScreenshotMaker = Ffmpeg(logger, inputVideoPath)
            elif Settings.MplayerPath and shutil.which(Settings.MplayerPath):
                self.InternalScreenshotMaker = Mplayer(logger, inputVideoPath)
        if self.InternalScreenshotMaker is None:
            raise PtpUploaderException("No screenshot tool found")

    def GetScaleSize(self):
        return self.InternalScreenshotMaker.ScaleSize

    # Returns with the URL of the uploaded image.
    def __TakeAndUploadScreenshot(self, timeInSeconds):
        with temporary_filename(".png") as outputPngPath:
            self.InternalScreenshotMaker.MakeScreenshotInPng(
                timeInSeconds, outputPngPath
            )

            if config.tools.oxipng.path and shutil.which(config.tools.oxipng.path):
                Oxipng.optimize_png(outputPngPath)
            elif Settings.ImageMagickConvertPath and shutil.which(
                Settings.ImageMagickConvertPath
            ):
                ImageMagick.OptimizePng(self.Logger, outputPngPath)

            imageUrl = ImageHost.upload(self.Logger, imagePath=outputPngPath)

        return imageUrl

    # Takes maximum five screenshots from the first 30% of the video.
    # Returns with the URLs of the uploaded images.
    def TakeAndUploadScreenshots(
        self, outputImageDirectory, durationInSec, numberOfScreenshotsToTake
    ):
        urls = []

        if numberOfScreenshotsToTake > config.uploader.max_screenshots:
            numberOfScreenshotsToTake = config.uploader.max_screenshots

        for i in range(numberOfScreenshotsToTake):
            position = 0.10 + (i * 0.05)
            urls.append(self.__TakeAndUploadScreenshot(int(durationInSec * position)))

        return urls

    # We sort video files by their size (less than 50 MB difference is ignored) and by their name.
    # Sorting by name is needed to ensure that the screenshot is taken from the first video to avoid spoilers when a release contains multiple videos.
    # Sorting by size is needed to ensure that we don't take the screenshots from the sample or extras included.
    # Ignoring less than 50 MB difference is needed to make sure that CD1 will be sorted before CD2 even if CD2 is larger than CD1 by 49 MB.
    @staticmethod
    def SortVideoFiles(files):
        class SortItem:
            def __init__(self, path):
                self.Path = path
                self.LowerPath = str(path).lower()
                self.Size = os.path.getsize(path)

            @staticmethod
            def Compare(item1, item2):
                ignoreSizeDifference = 50 * 1024 * 1024
                sizeDifference = item1.Size - item2.Size
                if abs(sizeDifference) > ignoreSizeDifference:
                    if item1.Size > item2.Size:
                        return -1
                    else:
                        return 1

                if item1.LowerPath < item2.LowerPath:
                    return -1
                elif item1.LowerPath > item2.LowerPath:
                    return 1
                else:
                    return 0

        filesToSort = []
        for file in files:
            item = SortItem(file)
            filesToSort.append(item)

        filesToSort.sort(key=functools.cmp_to_key(SortItem.Compare))

        files = []
        for item in filesToSort:
            files.append(item.Path)

        return files



# Inner exceptions will be only supported in Python 3000...
# All this magic here looks gross


class PtpUploaderException(Exception):
    # Overloads:
    # - PtpUploaderException( message )
    # - PtpUploaderException( jobRunningState, message )
    def __init__(self, *args):
        if len(args) == 1:
            Exception.__init__(self, args[0])
            self.JobRunningState = 4  # Failed, see ReleaseInfo
        else:
            Exception.__init__(self, args[1])
            self.JobRunningState = args[0]


# We handle this exception specially to make it unrecoverable.
# This is needed because to many login attempts with bad user name or password could result in temporary ban.
class PtpUploaderInvalidLoginException(PtpUploaderException):
    def __init__(self, message):
        PtpUploaderException.__init__(self, message)
